; sysmets4.pb
; based on sysmets4.c by Charles Petzold, in PW5E
;
IncludePath "..\"
XIncludeFile "petzold.pbi"
IncludePath ""
XIncludeFile "sysmets.pbi"

Global app_name.s = "sysmets4", main_caption.s = "Get System Metrics No. 4"
Global hinstance = GetModuleHandle_(#NULL)

Procedure main_callback(hwin, message, wparam, lparam)
    Static char_width, caps_width, char_height, client_width, client_height, scroll_vert_pos, max_width
    Define hdc, i, tm.TEXTMETRIC, y, x, scroll_vert_pos, scroll_horz_pos, paint_beg, paint_end
    Define si.SCROLLINFO
    
    Select message
        Case #WM_CREATE 
            hdc = GetDC_(hwin)

            GetTextMetrics_(hdc, tm)
            char_width = tm\tmAveCharWidth
            If tm\tmPitchAndFamily & 1 
                caps_width = 3 * char_width/2
            Else
                caps_width = char_width
            EndIf
            char_height = tm\tmHeight + tm\tmExternalLeading

            ReleaseDC_(hwin, hdc)

            ; Save the width of the three columns
            max_width = 40 * char_width + 22 * caps_width

        Case #WM_SIZE
            client_width = LOWORD(lparam)
            client_height = HIWORD(lparam)
            
            ; Set vertical scroll bar range and page size
            si\cbSize = SizeOf(SCROLLINFO)
            si\fMask  = #SIF_RANGE | #SIF_PAGE
            si\nMin   = 0
            si\nMax   = #NUMLINES - 1
            si\nPage  = client_height / char_height
            SetScrollInfo_(hwin, #SB_VERT, si, #True)
    
            ; Set horizontal scroll bar range and page size
            si\cbSize = SizeOf(SCROLLINFO)
            si\fMask  = #SIF_RANGE | #SIF_PAGE
            si\nMin   = 0
            si\nMax   = 2 + max_width / char_width
            si\nPage  = client_width / char_width
            SetScrollInfo_(hwin, #SB_HORZ, si, #True)
            
        Case #WM_VSCROLL
            ; Get all the vertical scroll bar information
            si\cbSize = SizeOf(SCROLLINFO)
            si\fMask  = #SIF_ALL
            GetScrollInfo_(hwin, #SB_VERT, si)
    
            ; Save the position for comparison later on
            scroll_vert_pos = si\nPos
        
            Select LOWORD(wparam)
                Case #SB_TOP
                    si\nPos = si\nMin
                Case #SB_BOTTOM
                    si\nPos = si\nMax
                Case #SB_LINEUP
                    si\nPos = si\nPos - 1
                Case #SB_LINEDOWN
                    si\nPos = si\nPos + 1
                Case #SB_PAGEUP
                    si\nPos = si\nPos - si\nPage
                Case #SB_PAGEDOWN
                    si\nPos = si\nPos + si\nPage
                Case #SB_THUMBTRACK
                    si\nPos = si\nTrackPos
                EndSelect
                ; Set the position and then retrieve it.  Due to adjustments
                ;   by Windows it may not be the same as the value set.
                si\fMask = #SIF_POS;
                SetScrollInfo_(hwin, #SB_VERT, si, #True)
                GetScrollInfo_(hwin, #SB_VERT, si)
        
                ; If the position has changed, scroll the window and update it
                If si\nPos <> scroll_vert_pos
                    ScrollWindow_(hwin, 0, char_height *(scroll_vert_pos - si\nPos), #NULL, #NULL)
                        UpdateWindow_(hwin)
                EndIf
        Case #WM_HSCROLL
                ; Get all the horizontal scroll bar information
                si\cbSize = SizeOf(SCROLLINFO)
                si\fMask  = #SIF_ALL
        
                ; Save the position for comparison later on
                GetScrollInfo_(hwin, #SB_HORZ, si)
                scroll_horz_pos = si\nPos
        
                Select LOWORD(wparam)
                    Case #SB_LINELEFT
                        si\nPos = si\nPos - 1
                    Case #SB_LINERIGHT
                        si\nPos = si\nPos + 1
                    Case #SB_PAGELEFT
                        si\nPos = si\nPos - si\nPage
                    Case #SB_PAGERIGHT
                        si\nPos = si\nPos + si\nPage
                    Case #SB_THUMBPOSITION
                        si\nPos = si\nTrackPos
                EndSelect
                ; Set the position and then retrieve it.  Due to adjustments
                ;   by Windows it may not be the same as the value set.
                si\fMask = #SIF_POS
                SetScrollInfo_(hwin, #SB_HORZ, si, #True)
                GetScrollInfo_(hwin, #SB_HORZ, si)
                ; If the position has changed, scroll the window 
                If si\nPos <> scroll_horz_pos
                    ScrollWindow_(hwin, char_width *(scroll_horz_pos - si\nPos), 0, #NULL, #NULL)
                EndIf
        Case #WM_KEYDOWN
            Select wparam
                Case #VK_HOME
                    SendMessage_(hwin, #WM_VSCROLL, #SB_TOP, 0)
                Case #VK_END
                    SendMessage_(hwin, #WM_VSCROLL, #SB_BOTTOM, 0)
                Case #VK_PRIOR
                    SendMessage_(hwin, #WM_VSCROLL, #SB_PAGEUP, 0)
                Case #VK_NEXT
                    SendMessage_(hwin, #WM_VSCROLL, #SB_PAGEDOWN, 0)
                Case #VK_UP
                    SendMessage_(hwin, #WM_VSCROLL, #SB_LINEUP, 0)
                Case #VK_DOWN
                    SendMessage_(hwin, #WM_VSCROLL, #SB_LINEDOWN, 0)
                Case #VK_LEFT
                    SendMessage_(hwin, #WM_HSCROLL, #SB_PAGEUP, 0)
                Case #VK_RIGHT
                    SendMessage_(hwin, #WM_HSCROLL, #SB_PAGEDOWN, 0)
            EndSelect

        Case #WM_PAINT
            define ps.PAINTSTRUCT
            hdc = BeginPaint_(hwin, ps)

            ; Get vertical scroll bar position
            si\cbSize = SizeOf(SCROLLINFO)
            si\fMask  = #SIF_POS
            GetScrollInfo_(hwin, #SB_VERT, si)
            scroll_vert_pos = si\nPos
    
            ; Get horizontal scroll bar position
            GetScrollInfo_(hwin, #SB_HORZ, si)
            scroll_horz_pos = si\nPos
    
            ; Find painting limits
            paint_beg = max(0, scroll_vert_pos + ps\rcPaint\top / char_height)
            paint_end = min(#NUMLINES - 1, scroll_vert_pos + ps\rcPaint\bottom / char_height)
            
            For i = paint_beg to paint_end
                x = char_width *(1 - scroll_horz_pos)
                y = char_height *(i - scroll_vert_pos)

            
                text_out(hdc, x, y, sys_metrics(i)\name)
                text_out(hdc, x + 22 * caps_width, y, sys_metrics(i)\desc)

                SetTextAlign_(hdc, #TA_RIGHT | #TA_TOP)

                text_out(hdc, x + 22 * caps_width + 40 * char_width, y, RSet(StrU(GetSystemMetrics_(sys_metrics(i)\flag),#PB_Long), 5))
    
                SetTextAlign_(hdc, #TA_LEFT | #TA_TOP)
            Next i
          
            EndPaint_(hwin, ps)
        Case #WM_DESTROY
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect 

    ProcedureReturn 0 
EndProcedure 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf

    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        TranslateMessage_(msg)
        DispatchMessage_(msg)
    Wend

End msg\wparam