; typer.pb
; based on typer.c by Charles Petzold, in PW5E
;
IncludePath "..\"
XIncludeFile "petzold.pbi"

Global app_name.s = "typer", main_caption.s = "Typing Program"
Global hinstance = GetModuleHandle_(#NULL)

Macro BUFFERADDR(x,y) 
    (*buffer + (y * cch_buffer_width + x)*SizeOf(CHAR))
EndMacro

Global *buffer.c = #NULL

Procedure main_callback(hwin, message, wparam, lparam)
    Static client_width, client_height, id_char_set = #DEFAULT_CHARSET, cch_buffer_width, cch_buffer_height
    Static char_width, char_height, x_caret, y_caret, x, y, i
    Define hdc, char.c
    
    Select message
        Case #WM_INPUTLANGCHANGE, #WM_CREATE, #WM_SIZE
            If message <> #WM_SIZE
                If message = #WM_INPUTLANGCHANGE
                    id_char_set = wparam
                EndIf
                hdc = GetDC_(hwin)
        
                SelectObject_(hdc, CreateFont_(0, 0, 0, 0, 0, 0, 0, 0, id_char_set, 0, 0, 0, #FIXED_PITCH, #NULL))  
                            
                Define tm.TEXTMETRIC
                GetTextMetrics_(hdc, tm)
                char_width = tm\tmAveCharWidth
                char_height = tm\tmHeight
                
                DeleteObject_(SelectObject_(hdc, GetStockObject_(#SYSTEM_FONT)))
                
                ReleaseDC_(hwin, hdc)
            Else  ;message = #WM_SIZE
                client_width = LOWORD(lparam)
                client_height = HIWORD(lparam)
            EndIf

            ; calculate window size in characters
            cch_buffer_width = max(1, client_width / char_width)
            cch_buffer_height = max(1, client_height / char_height)
            
            ; allocate memory for buffer and clear it            
            If *buffer
                FreeMemory(*buffer)
            EndIf
    
            *buffer = AllocateMemory(cch_buffer_width * cch_buffer_height * SizeOf(CHAR))
            
            For y = 0 to cch_buffer_height-1
                For x = 0 to cch_buffer_width-1
                    PokeC(BUFFERADDR(x,y), ' ')
                Next x
            Next y    

            ; set caret to upper left corner
            x_caret = 0
            y_caret = 0
                    
            If hwin = GetFocus_()
                SetCaretPos_(x_caret * char_width, y_caret * char_height)
            EndIf
            InvalidateRect_(hwin, #NULL, #True)
        Case #WM_SETFOCUS
            ; create and show the caret
            CreateCaret_(hwin, #NULL, char_width, char_height)
            SetCaretPos_(x_caret * char_width, y_caret * char_height)
            ShowCaret_(hwin)
        Case #WM_KILLFOCUS
            ; hide and destroy the caret    
            HideCaret_(hwin)
            DestroyCaret_()
        Case #WM_KEYDOWN
            Select wparam
                Case #VK_HOME
                    x_caret = 0
                Case #VK_END
                    x_caret = cch_buffer_width - 1
                Case #VK_PRIOR
                    y_caret = 0
                Case #VK_NEXT
                    y_caret = cch_buffer_height - 1
                Case #VK_LEFT
                    x_caret = max(x_caret - 1, 0)
                Case #VK_RIGHT
                    x_caret = min(x_caret + 1, cch_buffer_width - 1)
                Case #VK_UP
                    y_caret = max(y_caret - 1, 0)
                Case #VK_DOWN
                    y_caret = min(y_caret + 1, cch_buffer_height - 1)
                Case #VK_DELETE
                    For x = x_caret to cch_buffer_width-2
                        PokeC(BUFFERADDR(x, y_caret), PeekC(BUFFERADDR(x + 1, y_caret)))
                    Next x
                    
                    PokeC(BUFFERADDR(cch_buffer_width - 1, y_caret), ' ')

                    HideCaret_(hwin)
                    hdc = GetDC_(hwin)

                    SelectObject_(hdc, CreateFont_(0, 0, 0, 0, 0, 0, 0, 0, id_char_set, 0, 0, 0, #FIXED_PITCH, #NULL))

                    TextOut_(hdc, x_caret * char_width, y_caret * char_height, PeekS(BUFFERADDR(x_caret, y_caret)), cch_buffer_width - x_caret)

                    DeleteObject_(SelectObject_(hdc, GetStockObject_(#SYSTEM_FONT)))
                    ReleaseDC_(hwin, hdc)
                    ShowCaret_(hwin)
            EndSelect
            SetCaretPos_(x_caret * char_width, y_caret * char_height)
        Case #WM_CHAR
            For i = 0 to LOWORD(lparam)-1
                Select wparam
                    Case 8            ; backspace
                        If x_caret > 0
                            x_caret-1
                            SendMessage_(hwin, #WM_KEYDOWN, #VK_DELETE, 1)
                        EndIf
                    Case 9            ; tab
                        Repeat
                            SendMessage_(hwin, #WM_CHAR, ' ', 1)
                        Until x_caret % 8 = 0
                    Case 10            ; line feed
                        y_caret + 1
                        If y_caret = cch_buffer_height
                            y_caret = 0
                        EndIf
                    Case 13            ; carriage return
                        x_caret = 0
                        y_caret + 1
                        If y_caret = cch_buffer_height
                            y_caret = 0
                        EndIf
                    Case 27           ; escape
                        For y = 0 to cch_buffer_height-1
                            For x = 0 to cch_buffer_width-1
                                PokeC(BUFFERADDR(x, y), ' ')
                            Next x
                        Next y
                        x_caret = 0
                        y_caret = 0
                            
                        InvalidateRect_(hwin, #NULL, #False)
                    Default:              ; character codes
                        PokeC(BUFFERADDR(x_caret, y_caret), wparam)
                        
                        HideCaret_(hwin)
                        hdc = GetDC_(hwin)
                
                        SelectObject_(hdc, CreateFont_(0, 0, 0, 0, 0, 0, 0, 0, id_char_set, 0, 0, 0, #FIXED_PITCH, #NULL))
        
                        TextOut_(hdc, x_caret * char_width, y_caret * char_height, PeekS(BUFFERADDR(x_caret, y_caret)), 1)
        
                        DeleteObject_(SelectObject_(hdc, GetStockObject_(#SYSTEM_FONT)))
                        ReleaseDC_(hwin, hdc)
                        ShowCaret_(hwin)
        
                        x_caret + 1
                        If x_caret = cch_buffer_width
                            x_caret = 0
                            
                            y_caret + 1
                            If y_caret = cch_buffer_height
                                y_caret = 0
                            EndIf
                        EndIf
                EndSelect
            Next i
            
            SetCaretPos_(x_caret * char_width, y_caret * char_height)
        Case #WM_PAINT
            define ps.PAINTSTRUCT
            hdc = BeginPaint_(hwin, ps)
            
            SelectObject_(hdc, CreateFont_(0, 0, 0, 0, 0, 0, 0, 0, id_char_set, 0, 0, 0, #FIXED_PITCH, #NULL))
                            
            For y = 0 to cch_buffer_height-1
                TextOut_(hdc, 0, y * char_height, PeekS(BUFFERADDR(0,y)), cch_buffer_width)
            Next y
            DeleteObject_(SelectObject_(hdc, GetStockObject_(#SYSTEM_FONT)))
          
            EndPaint_(hwin, ps)
        Case #WM_DESTROY
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect 

    ProcedureReturn 0 
EndProcedure

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf

    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        TranslateMessage_(msg)
        DispatchMessage_(msg)
    Wend

End msg\wparam