; checker4.pb
; based on checker4.c by Charles Petzold, in PW5E
;
IncludePath "..\"
XIncludeFile "petzold.pbi"

Global app_name.s = "checker4", main_caption.s = "Checker4 Mouse Hit-Test Demo"
Global hinstance = GetModuleHandle_(#NULL)


#DIVISIONS = 5
Global id_focus = 0, child_class_name.s = "Checker4_Child"

Procedure childWinProc(hwin, message, wparam, lparam)
    Define hdc, ps.PAINTSTRUCT, rect.RECT
    
    Select message
        Case #WM_CREATE 
            SetWindowLongPtr_(hwin, 0, 0)   ; on/off flag
        Case #WM_KEYDOWN, #WM_LBUTTONDOWN
            ; Send most key presses to the hparent window
            If message = #WM_KEYDOWN And (wparam <> #VK_RETURN And wparam <> #VK_SPACE)
                SendMessage_(GetParent_(hwin), message, wparam, lparam)
            Else 
                SetWindowLongPtr_(hwin, 0, 1 ! GetWindowLongPtr_(hwin, 0))
                SetFocus_(hwin)
                InvalidateRect_(hwin, #NULL, #FALSE)
            EndIf 
        Case #WM_SETFOCUS, #WM_KILLFOCUS
            ; For focus messages, invalidate the window for repaint
            If message <> #WM_KILLFOCUS 
                id_focus = GetWindowLongPtr_(hwin, #GWL_ID)
            EndIf  
            InvalidateRect_(hwin, #NULL, #TRUE)
        Case #WM_PAINT 
            hdc = BeginPaint_(hwin, ps)
            
            GetClientRect_(hwin, rect)
            Rectangle_(hdc, 0, 0, rect\right, rect\bottom)
    
            ; Draw the "x" mark            
            If GetWindowLongPtr_(hwin, 0)
                line_to(hdc, 0,           0, rect\right, rect\bottom)
                line_to(hdc, 0, rect\bottom, rect\right,           0)
            EndIf 
    
            ; Draw the "focus" rectangle
            If hwin = GetFocus_()
                rect\left   + rect\right / 10
                rect\right  - rect\left
                rect\top    + rect\bottom / 10
                rect\bottom - rect\top
    
                SelectObject_(hdc, GetStockObject_(#NULL_BRUSH))
                SelectObject_(hdc, CreatePen_(#PS_DASH, 0, 0))
                Rectangle_(hdc, rect\left, rect\top, rect\right, rect\bottom)
                DeleteObject_(SelectObject_(hdc, GetStockObject_(#BLACK_PEN)))
            EndIf 
    
            EndPaint_(hwin, ps)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)    
    EndSelect 

    ProcedureReturn 0    
EndProcedure  


Procedure main_callback(hwin, message, wparam, lparam)
    Static Dim hchild(#DIVISIONS-1, #DIVISIONS-1)
    Define block_width, block_height, x, y

    Select message
        Case #WM_SIZE
            Define l=LOWORD(lparam), h = HIWORD(lparam) 
            block_width = Int(l / #DIVISIONS)
            block_height = Int(h / #DIVISIONS)
            
            For x = 0 To #DIVISIONS - 1
                For y = 0 To #DIVISIONS - 1
                    MoveWindow_(hchild(x,y), x * block_width, y * block_height, block_width, block_height, #TRUE)
                Next y
            Next x
        Case #WM_LBUTTONDOWN 
            MessageBeep_(0)
        Case #WM_SETFOCUS ; On set-focus message, set focus to child window
            SetFocus_(GetDlgItem_(hwin, id_focus))
        Case #WM_KEYDOWN  ; On key-down message, possibly change the focus window
            x = id_focus & $FF
            y = id_focus >> 8
            
            Select wparam
                Case #VK_UP     : y-1
                Case #VK_DOWN   : y+1
                Case #VK_LEFT   : x-1
                Case #VK_RIGHT  : x+1
                Case #VK_HOME   : x = y = 0
                Case #VK_END    : x = y = #DIVISIONS - 1
                Default         : ProcedureReturn 0
            EndSelect  
            
            x =(x + #DIVISIONS) % #DIVISIONS
            y =(y + #DIVISIONS) % #DIVISIONS
            
            id_focus = y << 8 | x
            
            SetFocus_(GetDlgItem_(hwin, id_focus))

        Case #WM_CREATE
            Define child_class.WNDCLASS
            With child_class
                \lpfnWndProc = @childWinProc()
                \cbWndExtra = SizeOf(INTEGER)
                \hinstance = hinstance
                \lpszClassName = @child_class_name
                \style = #CS_HREDRAW | #CS_VREDRAW
                \hcursor = LoadCursor_(0, #IDC_ARROW)
                \hbrBackground = #COLOR_WINDOW + 1
            EndWith
            RegisterClass_(child_class)

            For x = 0 to #DIVISIONS-1
                For y = 0 to #DIVISIONS-1
                    hchild(x,y) = CreateWindowEx_(#NULL,@child_class_name,#NULL, #WS_CHILDWINDOW | #WS_VISIBLE,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT, hwin, y << 8 | x,hinstance,#NULL)
                Next y
            Next x    

        Case #WM_DESTROY
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect 

    ProcedureReturn 0 
EndProcedure 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf

    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        TranslateMessage_(msg)
        DispatchMessage_(msg)
    Wend

End msg\wparam