; cliptext.pb
; based on cliptext.c by Charles Petzold, in PW5E
;
IncludePath "..\"
XIncludeFile "petzold.pbi"
IncludePath ""
XIncludeFile "cliptext.pbi"

Global app_name.s = "cliptext", main_caption.s = "Clipboard Text Transfers - ", default_text.s = "Default Text - "
Global hinstance = GetModuleHandle_(#NULL)


CompilerIf #PB_Compiler_Unicode
    #CF_TCHAR = #CF_UNICODETEXT
    main_caption + "Unicode Version"
    default_text + "Unicode Version"
CompilerElse
    #CF_TCHAR = #CF_TEXT
    main_caption + "ANSI Version"
    default_text + "ANSI Version"
CompilerEndIf


Procedure main_callback(hwin, message, wparam, lparam)
    Static *ptext
    Define hglobal, *pGlobal

    Select message
        Case #WM_INITMENUPOPUP
            If IsClipboardFormatAvailable_(#CF_TCHAR)
                Define f_enable = #MF_ENABLED
            Else
                f_enable = #MF_GRAYED
            EndIf
            EnableMenuItem_(wparam, #IDM_EDIT_PASTE, f_enable)

            If *ptext
                f_enable = #MF_ENABLED
            Else
                f_enable = #MF_GRAYED
            EndIf
            EnableMenuItem_(wparam, #IDM_EDIT_CUT,   f_enable)
            EnableMenuItem_(wparam, #IDM_EDIT_COPY,  f_enable)
            EnableMenuItem_(wparam, #IDM_EDIT_CLEAR, f_enable)
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
        Case #WM_CREATE
            SendMessage_(hwin, #WM_COMMAND, #IDM_EDIT_RESET, 0)
        Case #WM_COMMAND
            Define id = LOWORD(wparam)
            Select id
                Case #IDM_EDIT_PASTE
                    OpenClipboard_(hwin)
                    hglobal = GetClipboardData_(#CF_TCHAR)
                    If hglobal
                        *pGlobal = GlobalLock_(hglobal)

                        If *ptext
                            FreeMemory(*ptext)
                            *ptext = #NULL
                        EndIf
                        *ptext = AllocateMemory(GlobalSize_(hglobal))

                        lstrcpy_(*ptext, *pGlobal)
                        GlobalUnlock_(hglobal)
                        InvalidateRect_(hwin, #NULL, #TRUE)
                    EndIf
                    CloseClipboard_()
                Case #IDM_EDIT_CUT, #IDM_EDIT_COPY, #IDM_EDIT_CLEAR
                    If id <> #IDM_EDIT_CLEAR
                        If Not *ptext
                            ProcedureReturn 0
                        EndIf

                        hglobal = GlobalAlloc_(#GHND | #GMEM_SHARE, (lstrlen_(*ptext) + 1) * SizeOf(CHAR))
                        *pGlobal = GlobalLock_(hglobal)
                        lstrcpy_(*pGlobal, *ptext)
                        GlobalUnlock_(hglobal)

                        OpenClipboard_(hwin)
                        EmptyClipboard_()
                        SetClipboardData_(#CF_TCHAR, hglobal)
                        CloseClipboard_()
                    Endif

                    If id <> #IDM_EDIT_COPY
                        If *ptext
                            FreeMemory(*ptext)
                            *ptext = #NULL
                        EndIf
                        InvalidateRect_(hwin, #NULL, #TRUE)
                    EndIf
                Case #IDM_EDIT_RESET
                    If *ptext
                        FreeMemory(*ptext)
                        *ptext = #NULL
                    EndIf

                    *ptext = AllocateMemory((Len(default_text) + 1) * SizeOf(CHAR))
                    lstrcpy_(*ptext, @default_text)
                    InvalidateRect_(hwin, #NULL, #TRUE)
                Default
                    ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
            EndSelect
        Case #WM_PAINT
            Define ps.PAINTSTRUCT, rect.RECT
            Define hdc = BeginPaint_(hwin, ps)
            
            GetClientRect_(hwin, rect)

            If *ptext <> #NULL
                DrawText_(hdc, *ptext, -1, rect, #DT_EXPANDTABS | #DT_WORDBREAK)
            EndIf

            EndPaint_(hwin, ps)
        Case #WM_DESTROY
            If *ptext
                FreeMemory(*ptext)
            EndIf
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect 

    ProcedureReturn 0 
EndProcedure 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
            \lpszMenuName = @app_name
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf
        
    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define hAccel = LoadAccelerators_(hinstance, app_name)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        If Not TranslateAccelerator_(hwin, hAccel, msg)
            TranslateMessage_(msg)
            DispatchMessage_(msg)
        EndIf
    Wend

End msg\wparam