;   PopFile.pbi -- Popup Editor File Functions

Define ofn.OPENFILENAME

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Creating file filters in PureBasic code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; THIS WON"T WORK:
; Define fileFilters.s = "Text Files(*.TXT)" + "*.txt" + "ASCII Files(*.ASC)" + " *.asc" + "All Files(*.*)" + "*.*" +Chr(0)+Chr(0)
;       The strings get merged and the nulls thrown away.
; THIS WON"T WORK EITHER:
; Define fileFilters.s = "Text Files(*.TXT)" + Chr(0) + "*.txt" + Chr(0) + "ASCII Files(*.ASC)" + Chr(0) + " *.asc" + Chr(0) + "All Files(*.*)" + Chr(0) + "*.*" +Chr(0)+Chr(0)
;       The string's original nulls now get kept PLUS the Chr(0) nulls get added,
;       so the list is terminated (double-nulled) immediately after the first
;       string.

; THIS DOES THE TRICK:
DataSection
    ; PureBasic's string handling puts in the null terminators we need
    custom_file_filters: 
    Data.s "Text Files(*.TXT)" 
    Data.s "*.txt"
    Data.s "ASCII Files(*.ASC)"
    Data.s " *.asc"
    Data.s "All Files(*.*)"
    Data.s "*.*"
    Data.c 0      ; Filter list should end with an extra null terminator.
                  ; Note the data type: PureBasic will make it byte or word
                  ; depending on compilation mode.
EndDataSection
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure pop_file_initialize(hwin)
    Shared ofn

    ofn\lStructSize      = SizeOf(OPENFILENAME)
    ofn\hwndOwner        = hwin
    ofn\lpstrFilter      = ?custom_file_filters 
    ofn\nMaxFile        = #MAX_PATH
    ofn\nMaxFileTitle    = #MAX_PATH
    ofn\lpstrDefExt      = @"txt"
EndProcedure

Procedure pop_file_open_dlg(hwin, *file_name, *title_name)
    Shared ofn

    ofn\hwndOwner        = hwin
    ofn\lpstrFile        = *file_name
    ofn\lpstrFileTitle    = *title_name
    ofn\Flags           = #OFN_HIDEREADONLY | #OFN_CREATEPROMPT
    
    ProcedureReturn GetOpenFileName_(ofn)
EndProcedure

Procedure pop_file_save_dlg(hwin, *file_name, *title_name)
    Shared ofn

    ofn\hwndOwner        = hwin
    ofn\lpstrFile        = *file_name
    ofn\lpstrFileTitle    = *title_name
    ofn\Flags           = #OFN_OVERWRITEPROMPT
    
    ProcedureReturn GetSaveFileName_(ofn)
EndProcedure

Procedure pop_file_read(hedit, *file_name)
    Define cb_read.l, i, file_length, unicode_test, *b_buffer.b, *ptext.b, *b_conv.b

    ; Open the file.
    Define hfile = CreateFile_(*file_name, #GENERIC_READ, #FILE_SHARE_READ, #NULL, #OPEN_EXISTING, 0, #NULL)
    If #INVALID_HANDLE_VALUE = hfile 
        ProcedureReturn #FALSE
    EndIf

    ; Get file size in bytes and allocate memory for read.
    ; Add an extra two bytes for zero termination.
    file_length = GetFileSize_(hfile, #NULL)  ;; this assumes you won't try to open a file so big it takes
                                               ;; more than a dword to hold its size
    *b_buffer = AllocateMemory(file_length + 2)

    ; Read file and put terminating zeros at end.
    ReadFile_(hfile, *b_buffer, file_length, @cb_read, #NULL)
    CloseHandle_(hfile)
    PokeB(*b_buffer + file_length, 0)
    PokeB(*b_buffer + file_length+1, 0)

    ; Test to see if the text is Unicode
    ; IsTextUnicode() apparently returns non-zero for little-endian unicode,
    ; but not for big-endian! Nothing in the docs about this.
    ; However, the test flags returned in pUniTest are set correctly, so
    ; you can get reliable results from *them* . I'm not sure about the
    ; other varieties of unicode; does the api return non-zero for them, or
    ; not? Anyway, it doesn't matter for this app, it only handles utf-16
    ; little and big endian anyway.
    unicode_test = #IS_TEXT_UNICODE_SIGNATURE | #IS_TEXT_UNICODE_REVERSE_SIGNATURE
    IsTextUnicode_(*b_buffer, file_length, @unicode_test)
    If unicode_test & (#IS_TEXT_UNICODE_SIGNATURE | #IS_TEXT_UNICODE_REVERSE_SIGNATURE)
        *ptext = *b_buffer + 2;
        file_length - 2

        If unicode_test & #IS_TEXT_UNICODE_REVERSE_SIGNATURE
            For i = 0 To Int(file_length/2-1)
                Define bySwap.b = PeekB(*ptext + 2*i)
                PokeB(*ptext + 2*i, PeekB(*ptext + 2*i + 1))
                PokeB(*ptext + 2*i+1, bySwap)
            Next i
        EndIf

        ; Allocate memory for possibly converted string
        *b_conv = AllocateMemory(file_length + 2)

        ; If the edit control is not Unicode, convert Unicode text to
        ; non-Unicode(ie, in general, wide character).
        CompilerIf #PB_Compiler_Unicode <> 1
                WideCharToMultiByte_(#CP_ACP, 0, *ptext, -1, *b_conv, file_length + 2, #NULL, #NULL)
        CompilerElse ; If the edit control is Unicode, just copy the string
                lstrcpy_(*b_conv, *ptext)
        CompilerEndIf

    Else ; the file is not Unicode
        *ptext = *b_buffer

        ; Allocate memory for possibly converted string.
        *b_conv = AllocateMemory(2 * file_length + 2)

        ; If the edit control is Unicode, convert ASCII text.
        CompilerIf #PB_Compiler_Unicode = 1
                MultiByteToWideChar_(#CP_ACP, 0, *ptext, -1, *b_conv, file_length + 1)
        CompilerElse ; If not, just copy buffer
                lstrcpy_(*b_conv, *ptext)
        CompilerEndIf
    EndIf
    
    SetWindowText_(hedit, PeekS(*b_conv))
    FreeMemory(*b_buffer)
    FreeMemory(*b_conv)
   
    ProcedureReturn #TRUE
EndProcedure

Procedure PopFileWrite(hedit, *file_name)
    Define cb_written.l, hfile, length, *chb.c, byte_order_word.w = $FEFF

    ; Open the file, creating it if necessary
    hfile = CreateFile_(*file_name, #GENERIC_WRITE, 0, #NULL, #CREATE_ALWAYS, 0, #NULL)
    If hfile = #INVALID_HANDLE_VALUE
        ProcedureReturn #FALSE
    EndIf

    ; Get the number of characters in the edit control and allocate
    ; memory for them.
    length = GetWindowTextLength_(hedit)
    *chb = AllocateMemory((length + 1) * SizeOf(CHAR))
    
    If Not *chb
        CloseHandle_(hfile)
        ProcedureReturn #FALSE
    EndIf

    ; If the edit control will return Unicode text, write the
    ; byte order mark to the file.
    CompilerIf #PB_Compiler_Unicode = 1
        WriteFile_(hfile, @byte_order_word, 2, @cb_written, #NULL)
    CompilerEndIf

    ; Get the edit buffer and write that out to the file.
    GetWindowText_(hedit, *chb, length + 1)
    WriteFile_(hfile, *chb, length * SizeOf(CHAR), @cb_written, #NULL)
    
    If (length * SizeOf(CHAR)) <> cb_written
        CloseHandle_(hfile)
        FreeMemory(*chb)
        ProcedureReturn #FALSE
    EndIf
    
    CloseHandle_(hfile)
    FreeMemory(*chb)
    
    ProcedureReturn #TRUE
EndProcedure
