;  PopFind.pbi -- Popup Editor Search and Replace Functions


#MAX_STRING_LEN = 256

Define fs_find_text.s{#MAX_STRING_LEN}, fs_repl_text.s{#MAX_STRING_LEN}

Procedure pop_find_find_dlg(hwin)
    Shared fs_find_text
    Static fr.FINDREPLACE    ; must be static for modeless dialog!!!
    
    fr\lStructSize     = SizeOf(FINDREPLACE)
    fr\hwndOwner       = hwin
    fr\Flags          = #FR_HIDEUPDOWN | #FR_HIDEMATCHCASE | #FR_HIDEWHOLEWORD
    fr\lpstrFindWhat   = @fs_find_text
    fr\wFindWhatLen    = #MAX_STRING_LEN
    
    ProcedureReturn FindText_(fr)
EndProcedure

Procedure pop_find_replace_dlg(hwin)
    Shared fs_find_text, fs_repl_text
    Static fr.FINDREPLACE  ; must be static for modeless dialog!!!
    
    fr\lStructSize     = SizeOf(FINDREPLACE)
    fr\hwndOwner       = hwin
    fr\Flags          = #FR_HIDEUPDOWN | #FR_HIDEMATCHCASE | #FR_HIDEWHOLEWORD
    fr\lpstrFindWhat    = @fs_find_text
    fr\lpstrReplaceWith = @fs_repl_text
    fr\wFindWhatLen    = #MAX_STRING_LEN
    fr\wReplaceWithLen  = #MAX_STRING_LEN
    
    ProcedureReturn ReplaceText_(fr)
EndProcedure

Procedure pop_find_find_text(hedit, *search_offset, *fr.FINDREPLACE)
    Define length, *doc
    
    ; Read in the edit document
    length = GetWindowTextLength_(hedit)

    *doc = AllocateMemory((length + 1) * Sizeof(CHAR))
    If *doc = #NULL
        ProcedureReturn #FALSE
    EndIf

    GetWindowText_(hedit, *doc, length + 1)
    Define doc.s = PeekS(*doc, length)
    FreeMemory(*doc)

    ; Search the document for the find string
    Define target.s = PeekS(*fr\lpstrFindWhat)
    Define found = FindString(doc, target, PeekL(*search_offset))

    If Not found
        ProcedureReturn #False   ; Return an error code if the string cannot be found
    EndIf 
    
    ; Select the found text
    SendMessage_(hedit, #EM_SETSEL, found-1, found+Len(target)-1)
    SendMessage_(hedit, #EM_SCROLLCARET, 0, 0)

    PokeL(*search_offset, found + Len(target))

    ProcedureReturn #TRUE
EndProcedure

Procedure pop_find_next_text(hedit, *search_offset)
    Shared fs_find_text
    Define fr.FINDREPLACE
    
    fr\lpstrFindWhat = @fs_find_text
    
    ProcedureReturn pop_find_find_text(hedit, *search_offset, fr)
EndProcedure

Procedure pop_find_replace_text(hedit, *search_offset, *fr.FINDREPLACE)

    ; Find the text
    If Not pop_find_find_text(hedit, *search_offset, *fr)
        ProcedureReturn #FALSE
    EndIf

    ; Replace it
    SendMessage_(hedit, #EM_REPLACESEL, 0, *fr\lpstrReplaceWith)
    
    ProcedureReturn #TRUE
EndProcedure

Procedure pop_find_valid_find()
    Shared fs_find_text
    ProcedureReturn  (Len(fs_find_text) <> 0)
EndProcedure
