; poppad.pb
; based on poppad.c by Charles Petzold, in PW5E
;
IncludePath "..\"
XIncludeFile "petzold.pbi"

Global app_name.s = "poppad", main_caption.s = "PopPad"
Define hdlg_modeless

#EDITID = 1
Macro UNTITLED :"(untitled)": EndMacro

Global hinstance = GetModuleHandle_(#NULL)

IncludePath ""
XIncludeFile "PopPad.pbi"
XIncludeFile "PopPrnt.pbi"
XIncludeFile "PopFont.pbi"
XIncludeFile "PopFind.pbi"
XIncludeFile "PopFile.pbi"

Procedure main_callback(hwin, message, wparam, lparam)
    Static f_need_save = #FALSE, hedit, offset, id_find_replace_msg
    Static fs_file_name.s{#MAX_PATH}, fs_title_name.s{#MAX_PATH}
    Define sel_beg, sel_End, f_enable, *fr.FINDREPLACE
    Shared hdlg_modeless
    
    Select message
        Case #WM_SIZE
            MoveWindow_(hedit, 0, 0, LOWORD(lparam), HIWORD(lparam), #TRUE)
        Case #WM_SETFOCUS
            SetFocus_(hedit)
        Case #WM_INITMENUPOPUP
            Select lparam
                Case 1       ; Edit hmenu
                    ; Enable Undo if edit control can do it
                    If SendMessage_(hedit, #EM_CANUNDO, 0, 0)
                        Define enable_state = #MF_ENABLED
                    Else
                        enable_state = #MF_GRAYED
                    EndIf
                    EnableMenuItem_( wparam, #IDM_EDIT_UNDO, enable_state)

                    ; Enable Paste if text is in the clipboard
                    If IsClipboardFormatAvailable_(#CF_TEXT)
                        enable_state = #MF_ENABLED
                    Else
                        enable_state = #MF_GRAYED
                    EndIf
                    EnableMenuItem_(wparam, #IDM_EDIT_PASTE, enable_state)

                    ; Enable Cut, Copy, and Del if text is selected
                    SendMessage_(hedit, #EM_GETSEL, @sel_beg, @sel_End)

                    If sel_beg <> sel_End 
                        f_enable = #MF_ENABLED
                    Else
                        f_enable = #MF_GRAYED
                    EndIf

                    EnableMenuItem_(wparam, #IDM_EDIT_CUT,   f_enable)
                    EnableMenuItem_(wparam, #IDM_EDIT_COPY,  f_enable)
                    EnableMenuItem_(wparam, #IDM_EDIT_CLEAR, f_enable)
                Case 2       ; Search hmenu

                    ; Enable Find, Next, and Replace if modeless
                    ;   dialogs are not already active
                    If hdlg_modeless = #NULL 
                        f_enable = #MF_ENABLED
                    Else
                        f_enable = #MF_GRAYED
                    EndIf

                    EnableMenuItem_(wparam, #IDM_SEARCH_FIND,    f_enable)
                    EnableMenuItem_(wparam, #IDM_SEARCH_NEXT,    f_enable)
                    EnableMenuItem_(wparam, #IDM_SEARCH_REPLACE, f_enable)
            EndSelect
        Case #WM_CREATE
            ; Create the edit control child window
            hedit = CreateWindow_(@WC_EDIT,#NULL,#WS_CHILD | #WS_VISIBLE | #WS_HSCROLL | #WS_VSCROLL | #WS_BORDER | #ES_LEFT | #ES_MULTILINE | #ES_NOHIDESEL | #ES_AUTOHSCROLL | #ES_AUTOVSCROLL,0,0,0,0,hwin,#EDITID,hinstance,#NULL)

            SendMessage_(hedit, #EM_LIMITTEXT, 32000, 0)

            ; Initialize common dialog box stuff
            pop_file_initialize(hwin)
            pop_font_initialize(hedit)

            id_find_replace_msg = RegisterWindowMessage_(@FINDMSGSTRING)

            do_caption(hwin, fs_title_name)
        Case #WM_COMMAND
             ; cheat sheet:
             ; code = 0, is hmenu message : code = 1, is accelerator message : otherwise is control message and code is notification code
            Define id = LOWORD(wparam), code = HIWORD(wparam), hctrl = lparam
            ; Messages from edit control
            If hctrl And id = #EDITID
                Select code
                    Case #EN_UPDATE
                        f_need_save = #TRUE
                        ProcedureReturn 0
                    Case #EN_ERRSPACE, #EN_MAXTEXT
                        MessageBox_(hwin, "Edit control out of space.", app_name, #MB_OK | #MB_ICONSTOP)
                        ProcedureReturn 0
                    Default
                        ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
                EndSelect
            EndIf

            Select id
                ; Messages from File hmenu
                Case #IDM_FILE_NEW
                    If f_need_save And  #IDCANCEL = ask_about_save(hwin, fs_title_name)
                        ProcedureReturn 0
                    EndIf
                    SetWindowText_(hedit, "")
                    fs_file_name  = ""
                    fs_title_name = ""
                    do_caption(hwin, fs_title_name)
                    f_need_save = #FALSE
                Case #IDM_FILE_OPEN
                    If f_need_save And #IDCANCEL = ask_about_save(hwin, fs_title_name)
                        ProcedureReturn 0
                    EndIf

                    If pop_file_open_dlg(hwin, @fs_file_name, @fs_title_name)
                        If Not pop_file_read(hedit, @fs_file_name)
                            ok_message(hwin, "Could not read file " + fs_title_name + "!")
                            fs_file_name = ""
                            fs_title_name = ""
                        EndIf
                    EndIf

                    do_caption(hwin, fs_title_name)
                    f_need_save = #FALSE
                Case #IDM_FILE_SAVE, #IDM_FILE_SAVE_AS
                    If id = #IDM_FILE_SAVE
                        If Len(fs_file_name)
                            If PopFileWrite(hedit, @fs_file_name)
                                f_need_save = #FALSE
                                ProcedureReturn 1
                            Else
                                ok_message(hwin, "Could not write file " + fs_title_name)
                                ProcedureReturn 0
                            EndIf
                        EndIf
                    EndIf
                    If pop_file_save_dlg(hwin, @fs_file_name, @fs_title_name)
                        do_caption(hwin, fs_title_name)

                        If PopFileWrite(hedit, @fs_file_name)
                            f_need_save = #FALSE
                            ProcedureReturn 1
                        Else
                            ok_message(hwin, "Could not write file " + fs_title_name)
                            ProcedureReturn 0
                        EndIf
                    EndIf
                Case #IDM_FILE_PRINT
                    If Not pop_prnt_print_file(hinstance, hwin, hedit, fs_title_name)
                        ok_message(hwin, "Could not print file " + fs_title_name)
                    EndIf
                Case #IDM_APP_EXIT
                    SendMessage_(hwin, #WM_CLOSE, 0, 0)

                    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Messages from Edit hmenu
                Case #IDM_EDIT_UNDO
                    SendMessage_(hedit, #WM_UNDO, 0, 0)
                Case #IDM_EDIT_CUT
                    SendMessage_(hedit, #WM_CUT, 0, 0)
                Case #IDM_EDIT_COPY
                    SendMessage_(hedit, #WM_COPY, 0, 0)
                Case #IDM_EDIT_PASTE:
                    SendMessage_(hedit, #WM_PASTE, 0, 0)
                Case #IDM_EDIT_CLEAR:
                    SendMessage_(hedit, #WM_CLEAR, 0, 0)
                Case #IDM_EDIT_SELECT_ALL:
                    SendMessage_(hedit, #EM_SETSEL, 0, -1)

                    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Messages from Search hmenu
                Case #IDM_SEARCH_FIND
                    SendMessage_(hedit, #EM_GETSEL, 0, @offset)
                    hdlg_modeless = pop_find_find_dlg(hwin)
                Case #IDM_SEARCH_NEXT
                    SendMessage_(hedit, #EM_GETSEL, 0, @offset)

                    If pop_find_valid_find()
                        pop_find_next_text(hedit, @offset)
                    Else
                        hdlg_modeless = pop_find_find_dlg(hwin)
                    EndIf
                Case #IDM_SEARCH_REPLACE
                    SendMessage_(hedit, #EM_GETSEL, 0, @offset)
                    hdlg_modeless = pop_find_replace_dlg(hwin)
                Case #IDM_FORMAT_FONT
                    If pop_font_choose_font(hwin)
                        pop_font_set_font(hedit)
                    EndIf

                    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Messages from Help hmenu
                Case #IDM_HELP
                    ok_message(hwin, "Help not yet implemented!")
                Case #IDM_APP_ABOUT
                    DialogBox_(hinstance, @"AboutBox", hwin, @about_callback())
                Default
                    ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
            EndSelect
        Case #WM_CLOSE
            If (Not f_need_save) Or (#IDCANCEL <> ask_about_save(hwin, fs_title_name))
                DestroyWindow_(hwin)
            EndIf
        Case #WM_QUERYENDSESSION
            If (Not f_need_save) Or  (#IDCANCEL <> ask_about_save(hwin, fs_title_name))
                ProcedureReturn 1
            EndIf
        Case #WM_DESTROY
            pop_font_deinitialize()
            PostQuitMessage_(0)
        Default
            ; Process "Find-Replace" messages
            If message = id_find_replace_msg
                *fr = lparam

                If *fr\Flags & #FR_DIALOGTERM
                    hdlg_modeless = #NULL
                EndIf

                If *fr\Flags & #FR_FINDNEXT
                    If Not pop_find_find_text(hedit, @offset, *fr)
                        ok_message(hwin, "Text not found!")
                        SetFocus_(hdlg_modeless)
                    EndIf
                EndIf

                If *fr\Flags & #FR_REPLACE or *fr\Flags & #FR_REPLACEALL
                    If Not pop_find_replace_text(hedit, @offset, *fr)
                        ok_message(hwin, "Text not found!")
                        SetFocus_(hdlg_modeless)
                    EndIf
                EndIf
                If *fr\Flags & #FR_REPLACEALL
                    While pop_find_replace_text(hedit, @offset, *fr)
                    Wend
                EndIf
            Else
                ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
            EndIf
    EndSelect 

    ProcedureReturn 0 
EndProcedure 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(hinstance, @app_name)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
            \lpszMenuName = @app_name
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf
        
    Define *s_CmdLine = GetCommandLine_()

    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,*s_CmdLine)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define hAccel = LoadAccelerators_(hinstance, @app_name)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        If hdlg_modeless = #NULL Or Not IsDialogMessage_(hdlg_modeless, @msg)
            If Not TranslateAccelerator_(hwin, hAccel, msg)
                TranslateMessage_(msg)
                DispatchMessage_(msg)
            EndIf
        EndIf
    Wend

End msg\wparam