;   PopPrnt.pbi -- Popup Editor Printing Functions


Define f_user_abort, hprint_dlg
Procedure print_dlg_callback(hdlg, msg, wparam, lparam)
    Shared f_user_abort, hprint_dlg
    Select (msg)
        Case #WM_INITDIALOG
            EnableMenuItem_(GetSystemMenu_(hdlg, #FALSE), #SC_CLOSE, #MF_GRAYED)
            ProcedureReturn #TRUE

        Case #WM_COMMAND :
            f_user_abort = #TRUE
            EnableWindow_(GetParent_(hdlg), #TRUE)
            DestroyWindow_(hdlg)
            hprint_dlg = #NULL
            ProcedureReturn #TRUE
    EndSelect
    ProcedureReturn #FALSE
EndProcedure

Procedure abort_proc(hdc_printer, code)
    Shared f_user_abort, hprint_dlg
    Define msg.MSG

    While (Not f_user_abort And PeekMessage_(msg, #NULL, 0, 0, #PM_REMOVE))
        If (Not hprint_dlg Or Not IsDialogMessage_(hprint_dlg, msg))
            TranslateMessage_(msg)
            DispatchMessage_(msg)
        EndIf
    Wend
    ProcedureReturn (Not f_user_abort)
EndProcedure

Procedure pop_prnt_print_file( hInst, hwin, hedit, title_name.s)
    Shared f_user_abort, hprint_dlg
    Define f_success, char_height, chars_per_line, lines_per_page, total_lines
    Define total_pages, page_num, line_num, temp_line_num, tm.TEXTMETRIC
    Define col_copy.w, no_col_copy.w
    Static pd.PRINTDLG, di.DOCINFO

    di\cbSize = SizeOf(DOCINFO)

    ; Invoke Print common dialog box
    pd\lStructSize        = SizeOf(PRINTDLG)
    pd\hwndOwner         = hwin
    pd\hDevMode          = #NULL
    pd\hDevNames         = #NULL
    pd\hDC              = #NULL
    pd\Flags            = #PD_ALLPAGES | #PD_COLLATE | #PD_RETURNDC | #PD_NOSELECTION
    pd\nFromPage         = 0
    pd\nToPage           = 0
    pd\nMinPage          = 0
    pd\nMaxPage          = 0
    pd\nCopies           = 1
    pd\hInstance         = #NULL
    pd\lCustData         = 0
    pd\lpfnPrintHook      = #NULL
    pd\lpfnSetupHook      = #NULL
    pd\lpPrintTemplateName = #NULL
    pd\lpSetupTemplateName = #NULL
    pd\hPrintTemplate     = #NULL
    pd\hSetupTemplate     = #NULL

    If (Not PrintDlg_(pd))
        ProcedureReturn #TRUE
    EndIf
    total_lines = SendMessage_(hedit, #EM_GETLINECOUNT, 0, 0)
    If total_lines = 0
        ProcedureReturn #TRUE
    EndIf

    ; Calculate necessary metrics for file
    GetTextMetrics_(pd\hDC, tm)
    char_height = tm\tmHeight + tm\tmExternalLeading

    chars_per_line = GetDeviceCaps_(pd\hDC, #HORZRES) / tm\tmAveCharWidth
    lines_per_page = GetDeviceCaps_(pd\hDC, #VERTRES) / char_height
    total_pages   =(total_lines + lines_per_page - 1) / lines_per_page

    ; Allocate a buffer for each line of text
    Define *ch_buffer = AllocateMemory(SizeOf(CHAR) * (chars_per_line + 1))

    ; Display the printing dialog box
    EnableWindow_(hwin, #FALSE)

    f_success   = #TRUE
    f_user_abort = #FALSE

    hprint_dlg = CreateDialog_(hInst, @"PrintDlgBox", hwin, @print_dlg_callback())

    SetDlgItemText_(hprint_dlg, #IDC_FILENAME, @title_name)
    SetAbortProc_(pd\hDC, @abort_proc())

    ; Start the document
    Define fs_job_name.s{64 + #MAX_PATH}
    GetWindowText_(hwin, @fs_job_name, SizeOf(fs_job_name))
    di\lpszDocName = @fs_job_name

    If StartDoc_(pd\hDC, @di) > 0
        ; Collation requires this loop and no_col_copy
        Define col_copy_max, no_col_copy_max
        If pd\Flags & #PD_COLLATE
            col_copy_max = pd\nCopies - 1
            no_col_copy_max = 0
        Else
            col_copy_max = 0
            no_col_copy_max = pd\nCopies - 1
        EndIf
        For col_copy = 0 To col_copy_max
            For page_num = 0  To total_pages - 1
                For no_col_copy = 0 To no_col_copy_max
                    ; Start the page
                    If StartPage_(pd\hDC) < 0
                        f_success = #FALSE
                        Break
                    EndIf

                    ; For each page, print the lines
                    For line_num = 0 To lines_per_page - 1
                        temp_line_num = lines_per_page * page_num + line_num

                        If temp_line_num > total_lines
                            break
                        EndIf
                        
                        ; Sticking a number in a string buffer looks odd, I know, so
                        ; here's the explanation from EM_GETLINE docs:
                        ; "The first word of the buffer specifies the maximum
                        ; number of characters that can be copied to the buffer."
                        PokeW(*ch_buffer, chars_per_line)

                        TextOut_(pd\hDC, 0, char_height * line_num, *ch_buffer, SendMessage_(hedit, #EM_GETLINE, temp_line_num, *ch_buffer))
                    Next line_num

                    If EndPage_(pd\hDC) < 0
                        f_success = #FALSE
                        Break
                    EndIf

                    If f_user_abort
                        Break
                    EndIf
                Next no_col_copy

                If (Not f_success Or f_user_abort)
                    Break
                EndIf
            Next page_num

            If (Not f_success Or f_user_abort)
                Break
            EndIf
        Next col_copy
    Else
        f_success = #FALSE
    EndIf
    
    If f_success
        EndDoc_(pd\hDC)
    EndIf

    If Not f_user_abort
        EnableWindow_(hwin, #TRUE)
        DestroyWindow_(hprint_dlg)
    EndIf

    FreeMemory(*ch_buffer)
    DeleteDC_(pd\hDC)

    ProcedureReturn (f_success And Not f_user_abort)
EndProcedure
