;
; sketch.pb
; based on sketch.C by Charles Petzold, in PW5E
IncludePath "..\"
XIncludeFile "petzold.pbi"

Global app_name.s = "sketch", main_caption.s = "Sketch"
Global hinstance = GetModuleHandle_(#NULL)

IncludePath ""
XIncludeFile "sketch.pbi"

Procedure main_callback(hwin, message, wparam, lparam)
    Static left_button_down, right_button_down
    Static hbitmap, hdc_mem, bitmap_width, bitmap_height, client_width, client_height, x_mouse, y_mouse

    Select message
        Case #WM_PAINT
            Define ps.PAINTSTRUCT
            Define hdc = BeginPaint_(hwin, ps)

            BitBlt_(hdc, 0, 0, client_width, client_height, hdc_mem, 0, 0, #SRCCOPY)

            EndPaint_(hwin, ps)
        Case #WM_LBUTTONDOWN
            If Not right_button_down
                SetCapture_(hwin)
            EndIf 
            x_mouse = LOWORD(lParam)
            y_mouse = HIWORD(lParam)
            left_button_down = #TRUE
        Case #WM_LBUTTONUP
            If left_button_down
                SetCapture_(#NULL)
            EndIf 
            left_button_down = #FALSE
        Case #WM_RBUTTONDOWN
            If Not left_button_down
                SetCapture_(hwin)
            EndIf 
            x_mouse = LOWORD(lParam)
            y_mouse = HIWORD(lParam)
            right_button_down = #TRUE
        Case #WM_RBUTTONUP
            If right_button_down 
                SetCapture_(#NULL)
            EndIf 
            
            right_button_down = #FALSE    
        Case #WM_MOUSEMOVE
            If Not left_button_down And Not right_button_down
                ProcedureReturn 0
            EndIf 
            hdc = GetDC_(hwin)
    
            If left_button_down
                Define pen = #BLACK_PEN
            Else
                pen = #WHITE_PEN
            EndIf 
            SelectObject_(hdc, GetStockObject_(pen))
    
            SelectObject_(hdc_mem, GetStockObject_(pen))
    
            MoveToEx_(hdc,    x_mouse, y_mouse, #NULL)
            MoveToEx_(hdc_mem, x_mouse, y_mouse, #NULL)
    
            x_mouse = LOWORD(lParam)
            y_mouse = HIWORD(lParam)
    
            LineTo_(hdc,    x_mouse, y_mouse)
            LineTo_(hdc_mem, x_mouse, y_mouse)
    
            ReleaseDC_(hwin, hdc)
        Case #WM_SIZE
            client_width = LOWORD(lParam)
            client_height = HIWORD(lParam)
        Case #WM_CREATE
            GetLargestDisplayMode(@bitmap_width, @bitmap_height)
    
            hdc = GetDC_(hwin)
            hbitmap = CreateCompatibleBitmap_(hdc, bitmap_width, bitmap_height)
            hdc_mem  = CreateCompatibleDC_(hdc)
            ReleaseDC_(hwin, hdc)
    
            If Not hbitmap  ; no memory for bitmap
                DeleteDC_(hdc_mem)
                ProcedureReturn -1
            EndIf 
    
            SelectObject_(hdc_mem, hbitmap)
            PatBlt_(hdc_mem, 0, 0, bitmap_width, bitmap_height, #WHITENESS)

        Case #WM_DESTROY
            DeleteDC_(hdc_mem)
            DeleteObject_(hbitmap)
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect

    ProcedureReturn 0
EndProcedure
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf
        
    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        TranslateMessage_(msg)
        DispatchMessage_(msg)
    Wend

End msg\wparam
