
#IDM_FONT_COUR = 101
#IDM_FONT_ARIAL = 102
#IDM_FONT_TIMES = 103
#IDM_HELP = 104
#IDM_EDIT_UNDO = 40005
#IDM_EDIT_CUT = 40006
#IDM_EDIT_COPY = 40007
#IDM_EDIT_PASTE = 40008
#IDM_EDIT_CLEAR = 40009
#IDM_FILE_NEW = 40010
#IDM_FILE_OPEN = 40011
#IDM_FILE_SAVE = 40012
#IDM_FILE_SAVE_AS = 40013


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; StretchBitmap: Scales bitmap to display resolution
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure StretchBitmap(hbmp1)
    Define bm1.BITMAP, bm2.BITMAP
    
    ; Get the width and height of a system font character
    Define char_width = LOWORD(GetDialogBaseUnits_())
    Define char_height = HIWORD(GetDialogBaseUnits_())

    ; Create 2 memory DCs compatible with the display
    Define hdc = CreateIC_("DISPLAY", #NULL, #NULL, #NULL)
    Define hdc_mem1 = CreateCompatibleDC_(hdc)
    Define hdc_mem2 = CreateCompatibleDC_(hdc)
    DeleteDC_(hdc)

    ; Get the dimensions of the bitmap to be stretched
    GetObject_(hbmp1, SizeOf(BITMAP), bm1)

    ; Scale these dimensions based on the system font size
    CopyMemory(bm1, bm2, SizeOf(BITMAP))
    bm2\bmWidth     =(char_width * bm2\bmWidth)  / 4
    bm2\bmHeight    =(char_height * bm2\bmHeight) / 8
    bm2\bmWidthBytes =((bm2\bmWidth + 15) / 16) * 2

    ; Create a new bitmap of larger size
    Define hbmp2 = CreateBitmapIndirect_(bm2)

    ; Select the bitmaps in the memory DCs and do a StretchBlt_
    SelectObject_(hdc_mem1, hbmp1)
    SelectObject_(hdc_mem2, hbmp2)

    StretchBlt_(hdc_mem2, 0, 0, bm2\bmWidth, bm2\bmHeight, hdc_mem1, 0, 0, bm1\bmWidth, bm1\bmHeight, #SRCCOPY)

    ; Clean up
    DeleteDC_(hdc_mem1)
    DeleteDC_(hdc_mem2)
    DeleteObject_(hbmp1)

    ProcedureReturn hbmp2
EndProcedure 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; AddHelpToSys: Adds bitmap Help item to system menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure AddHelpToSys(hinstance, hwin)
    Define hmenu = GetSystemMenu_(hwin, #FALSE)
    Define hbitmap = StretchBitmap(LoadBitmap_(hinstance, "BitmapHelp"))
    AppendMenu_(hmenu, #MF_BITMAP | #MF_MENUBARBREAK, #IDM_HELP, hbitmap)
EndProcedure 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; GetBitmapFont: Creates bitmaps with font names
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure GetBitmapFont(i)
    Define size.SIZE, tm.TEXTMETRIC
    Dim facename.s(2) 
    facename(0) = "Courier New"
    facename(1) = "Arial"
    facename(2) = "Times New Roman"
    
    
    Define hdc = CreateIC_("DISPLAY", #NULL, #NULL, #NULL)
    GetTextMetrics_(hdc, tm)
    
    Define hdc_mem = CreateCompatibleDC_(hdc)
    Define hfont  = CreateFont_(2 * tm\tmHeight, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, facename(i))

    hfont = SelectObject_(hdc_mem, hfont)
    GetTextExtentPoint32_(hdc_mem, facename(i), Len(facename(i)), size)
    
    Define hbitmap = CreateBitmap_(size\cx, size\cy, 1, 1, #NULL)
    SelectObject_(hdc_mem, hbitmap)
    
    text_out(hdc_mem, 0, 0, facename(i))
    
    DeleteObject_(SelectObject_(hdc_mem, hfont))
    DeleteDC_(hdc_mem)
    DeleteDC_(hdc)
    
    ProcedureReturn hbitmap
EndProcedure 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CreateMyMenu: Assembles menu from components
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure CreateMyMenu(hinstance)

    Define hmenu = CreateMenu_()

    Define hmenu_popup = LoadMenu_(hinstance, "MenuFile")
    Define hbitmap = StretchBitmap(LoadBitmap_(hinstance, "BitmapFile"))
    AppendMenu_(hmenu, #MF_BITMAP | #MF_POPUP, hmenu_popup, hbitmap)

    hmenu_popup = LoadMenu_(hinstance, "MenuEdit")
    hbitmap = StretchBitmap(LoadBitmap_(hinstance, "BitmapEdit"))
    AppendMenu_(hmenu, #MF_BITMAP | #MF_POPUP, hmenu_popup, hbitmap)
    
    hmenu_popup = CreateMenu_()
    
    Define i
    For i = 0 To 2
        hbitmap = GetBitmapFont(i)
        AppendMenu_(hmenu_popup, #MF_BITMAP, #IDM_FONT_COUR + i, hbitmap)
    Next i

    hbitmap = StretchBitmap(LoadBitmap_(hinstance, "BitmapFont"))
    AppendMenu_(hmenu, #MF_BITMAP | #MF_POPUP, hmenu_popup, hbitmap)

    ProcedureReturn hmenu
EndProcedure 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; DeleteAllBitmaps: Deletes all the bitmaps in the menu
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Procedure DeleteAllBitmaps(hwin)
    Define mii.MENUITEMINFO
 
    mii\cbsize = SizeOf(MENUITEMINFO)
    mii\fmask = #MIIM_SUBMENU | #MIIM_TYPE
     
    ; Delete Help bitmap on system menu
    Define hmenu = GetSystemMenu_(hwin, #FALSE)
    GetMenuItemInfo_(hmenu, #IDM_HELP, #FALSE, mii)
    DeleteObject_(mii\dwtypedata)

    ; Delete top-level menu bitmaps
    hmenu = GetMenu_(hwin)

    Define i
    For i = 0 To 2
        GetMenuItemInfo_(hmenu, i, #TRUE, mii)
        DeleteObject_(mii\dwTypeData)
    Next i

    ; Delete bitmap items on Font menu
    hmenu = mii\hSubMenu

    For i = 0 To 2
        GetMenuItemInfo_(hmenu, i, #TRUE, mii)
        DeleteObject_(mii\dwTypeData)
    Next i
EndProcedure 
