;
; bitmask.pb
; based on bitmask.C by Charles Petzold, in PW5E
IncludePath "..\"
XIncludeFile "petzold.pbi"

Global app_name.s = "bitmask", main_caption.s = "Bitmap Masking Demo"
Global hinstance = GetModuleHandle_(#NULL)

Procedure main_callback(hwin, message, wparam, lparam)
    Static hbmp_imag, hbmp_mask, client_width, client_height, bitmap_width, bitmap_height
    Define bitmap.BITMAP,  hdc_mem_imag, hdc_mem_mask, x, y

    Select message
        Case #WM_SIZE
            client_width = LOWORD(lParam)
            client_height = HIWORD(lParam)
        Case #WM_CREATE
            ; Load the original image and get its size
            hbmp_imag = LoadBitmap_(hinstance, "BITMASK")
            GetObject_(hbmp_imag, SizeOf(BITMAP), bitmap)
            bitmap_width = bitmap\bmWidth
            bitmap_height = bitmap\bmHeight
    
            ; Select the original image into a memory DC
            hdc_mem_imag  = CreateCompatibleDC_(#NULL)
            SelectObject_(hdc_mem_imag, hbmp_imag)
    
            ; Create the monochrome mask bitmap and memory DC
            hbmp_mask = CreateBitmap_(bitmap_width, bitmap_height, 1, 1, #NULL)
            hdc_mem_mask = CreateCompatibleDC_(#NULL)
            SelectObject_(hdc_mem_mask, hbmp_mask)
    
            ; Color the mask bitmap black with a white ellipse
            SelectObject_(hdc_mem_mask, GetStockObject_(#BLACK_BRUSH))
            Rectangle_(hdc_mem_mask, 0, 0, bitmap_width, bitmap_height)
            SelectObject_(hdc_mem_mask, GetStockObject_(#WHITE_BRUSH))
            Ellipse_(hdc_mem_mask, 0, 0, bitmap_width, bitmap_height)
    
            ; Mask the original image
            BitBlt_(hdc_mem_imag, 0, 0, bitmap_width, bitmap_height, hdc_mem_mask, 0, 0, #SRCAND)
    
            DeleteDC_(hdc_mem_imag)
            DeleteDC_(hdc_mem_mask)
        Case #WM_PAINT
            Define ps.PAINTSTRUCT
            Define hdc = BeginPaint_(hwin, ps)

            ; Select bitmaps into memory DCs
            hdc_mem_imag = CreateCompatibleDC_(hdc)
            SelectObject_(hdc_mem_imag, hbmp_imag)
    
            hdc_mem_mask = CreateCompatibleDC_(hdc)
            SelectObject_(hdc_mem_mask, hbmp_mask)
    
            ; Center image
            x =(client_width - bitmap_width) / 2
            y =(client_height - bitmap_height) / 2
                
            ; Do the BitBlts
            BitBlt_(hdc, x, y, bitmap_width, bitmap_height, hdc_mem_mask, 0, 0, $220326)
            BitBlt_(hdc, x, y, bitmap_width, bitmap_height, hdc_mem_imag, 0, 0, #SRCPAINT)
    
            DeleteDC_(hdc_mem_imag)
            DeleteDC_(hdc_mem_mask)

            EndPaint_(hwin, ps)
        Case #WM_DESTROY
            DeleteObject_(hbmp_imag)
            DeleteObject_(hbmp_mask)
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect

    ProcedureReturn 0
EndProcedure
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf
        
    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        TranslateMessage_(msg)
        DispatchMessage_(msg)
    Wend

End msg\wparam
