;
; bounce.pb
; based on bounce.C by Charles Petzold, in PW5E
;
IncludePath "..\"
XIncludeFile "petzold.pbi"

Global app_name.s = "bounce", main_caption.s = "Bouncing Ball"
Global hinstance = GetModuleHandle_(#NULL)
#ID_TIMER = 1

Procedure main_callback(hwin, message, wparam, lparam)
    Static hbitmap, client_width, client_height, x_center, y_center
    Static width, height, x_radius, y_radius, x_move, y_move 
    Static relative_pixel_width, relative_pixel_height
    Define hbrush, hdc, hdc_mem, scale

    Select message
        Case #WM_TIMER
            If Not hbitmap
                ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
            EndIf
            
            hdc = GetDC_(hwin)
            hdc_mem = CreateCompatibleDC_(hdc)
            SelectObject_(hdc_mem, hbitmap)
            
            BitBlt_(hdc, x_center - Int(width / 2), y_center - Int(height / 2), width, height, hdc_mem, 0, 0, #SRCCOPY)

            ReleaseDC_(hwin, hdc)
            DeleteDC_(hdc_mem)
            
            x_center + x_move
            y_center + y_move
            
            If ((x_center + x_radius >= client_width) Or(x_center - x_radius <= 0))
                x_move = -x_move
                MessageBeep_(#MB_OK)
            EndIf
            If ((y_center + y_radius >= client_height) Or(y_center - y_radius <= 0))
                y_move = -y_move
                MessageBeep_(#MB_OK)
            EndIf 
        Case #WM_SIZE
            client_width = LOWORD(lParam)
            client_height = HIWORD(lParam)
            x_center = Int(client_width / 2)
            y_center = Int(client_height / 2)

            scale = Int(min(client_width * relative_pixel_width, client_height * relative_pixel_height) / 16)
            
            x_radius = Int(scale / relative_pixel_width)
            y_radius = Int(scale / relative_pixel_height)
            
            x_move = max(1, Int(x_radius / 8))
            y_move = max(1, Int(y_radius / 8))
            
            width = 2 *(x_radius + x_move)
            height = 2 *(y_radius + y_move)
            
            If hbitmap
                DeleteObject_(hbitmap)
            EndIf 
            hdc = GetDC_(hwin)
            hdc_mem = CreateCompatibleDC_(hdc)
            hbitmap = CreateCompatibleBitmap_(hdc, width, height)

            ReleaseDC_(hwin, hdc)
            
            SelectObject_(hdc_mem, hbitmap)
            Rectangle_(hdc_mem, -1, -1, width + 1, height + 1)
            
            hbrush = CreateHatchBrush_(#HS_DIAGCROSS, 0)
            SelectObject_(hdc_mem, hbrush)
            SetBkColor_(hdc_mem, RGB(255, 0, 255))
            Ellipse_(hdc_mem, x_move, y_move, width - x_move, height - y_move)
            DeleteDC_(hdc_mem)
            DeleteObject_(hbrush)
        Case #WM_CREATE
            hdc = GetDC_(hwin)
            relative_pixel_width = GetDeviceCaps_(hdc, #ASPECTX)
            relative_pixel_height = GetDeviceCaps_(hdc, #ASPECTY)
            ReleaseDC_(hwin, hdc)
            
            SetTimer_(hwin, #ID_TIMER, 10, #NULL)

        Case #WM_DESTROY
            If hbitmap
                DeleteObject_(hbitmap)
            EndIf 
            KillTimer_(hwin, #ID_TIMER)
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect

    ProcedureReturn 0
EndProcedure
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf
        
    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        TranslateMessage_(msg)
        DispatchMessage_(msg)
    Wend

End msg\wparam