;
; blowup.pb
; based on blowup.c by Charles Petzold, in PW5E
; PETZOLD'S INSTRUCTIONS FOR USING THE PROG:
; Press the left mouse button in BLOWUP's client area, and keep the left 
; button held down. The mouse cursor changes to a crosshair.
;
; Still holding the left button down, move the mouse cursor anywhere on the 
; screen. Position the mouse cursor at the upper left corner of the rectangular 
; area you want to capture.
;
; Still holding the left button down, press the right mouse button and drag the 
; mouse to the lower right corner of the rectangular area you want to capture. 
; Release the left and right mouse buttons. (The order in which you release the 
; buttons doesn't matter.)
IncludePath "..\"
XIncludeFile "petzold.pbi"

IncludePath ""
XIncludeFile "blowup.pbi"

Global app_name.s = "blowup", main_caption.s = "Blow-Up Mouse Demo"
Global hinstance = GetModuleHandle_(#NULL)

Procedure main_callback(hwin, message, wparam, lparam)
    Static capturing, blocking, hbitmap, hscreen, pt_beg.POINT, pt_end.POINT
    Define hbmp_clip, hdc_mem, hdc, short.w  

    Select message
        Case #WM_LBUTTONDOWN
            If Not capturing
                hscreen = GetDesktopWindow_()
                If LockWindowUpdate_(hscreen)
                    capturing = #TRUE
                    SetCapture_(hwin)
                    SetCursor_(LoadCursor_(#NULL, #IDC_CROSS))
                Else
                    MessageBeep_(0)
                EndIf 
            EndIf 
        Case #WM_RBUTTONDOWN
            If capturing
                blocking = #TRUE
                short = LOWORD(lParam)  : pt_beg\x = short
                short = HIWORD(lParam)  : pt_beg\y = short
                CopyMemory(pt_beg, pt_end, SizeOf(POINT))
                InvertBlock(hscreen, hwin, pt_beg, pt_end)
            EndIf
        Case #WM_MOUSEMOVE
            If blocking
                InvertBlock(hscreen, hwin, pt_beg, pt_end)
                short = LOWORD(lParam)  : pt_end\x = short
                short = HIWORD(lParam)  : pt_end\y = short
                InvertBlock(hscreen, hwin, pt_beg, pt_end)
            EndIf 
        Case #WM_LBUTTONUP, #WM_RBUTTONUP
            If blocking
                InvertBlock(hscreen, hwin, pt_beg, pt_end)
                short = LOWORD(lParam)  : pt_end\x = short
                short = HIWORD(lParam)  : pt_end\y = short
    
                If hbitmap
                    DeleteObject_(hbitmap)
                    hbitmap = #NULL
                EndIf 

                hdc = GetDC_(hwin)
                hdc_mem = CreateCompatibleDC_(hdc)
                hbitmap = CreateCompatibleBitmap_(hdc, absi(pt_end\x - pt_beg\x), absi(pt_end\y - pt_beg\y))
                SelectObject_(hdc_mem, hbitmap)

                StretchBlt_(hdc_mem, 0, 0, absi(pt_end\x - pt_beg\x), absi(pt_end\y - pt_beg\y), hdc, pt_beg\x, pt_beg\y, pt_end\x - pt_beg\x, pt_end\y - pt_beg\y, #SRCCOPY)

                DeleteDC_(hdc_mem)
                ReleaseDC_(hwin, hdc)
                InvalidateRect_(hwin, #NULL, #TRUE)
            EndIf 
            If blocking Or capturing
                capturing = #FALSE
                blocking = #FALSE 
                SetCursor_(LoadCursor_(#NULL, #IDC_ARROW))
                ReleaseCapture_()
                LockWindowUpdate_(#NULL)
            EndIf
        Case #WM_INITMENUPOPUP
            Define enable
            If IsClipboardFormatAvailable_(#CF_BITMAP)
                enable =  #MF_ENABLED
            Else
                enable = #MF_GRAYED
            EndIf 

            EnableMenuItem_(wParam, #IDM_EDIT_PASTE, enable)
    
            If hbitmap
                enable = #MF_ENABLED 
            Else
                enable = #MF_GRAYED
            EndIf 
            
            EnableMenuItem_(wParam, #IDM_EDIT_CUT,    enable)
            EnableMenuItem_(wParam, #IDM_EDIT_COPY,   enable)
            EnableMenuItem_(wParam, #IDM_EDIT_DELETE, enable)
        Case #WM_COMMAND
            Define id = LOWORD(wParam)
            Select id
                Case #IDM_EDIT_CUT, #IDM_EDIT_COPY, #IDM_EDIT_DELETE
                    If id <> #IDM_EDIT_DELETE
                        If hbitmap
                            hbmp_clip = CopyBitmap(hbitmap)
                            OpenClipboard_(hwin)
                            EmptyClipboard_()
                            SetClipboardData_(#CF_BITMAP, hbmp_clip)
                        EndIf 
                        If id = #IDM_EDIT_COPY
                            ProcedureReturn 0
                        EndIf 
                    EndIf 

                    ;; fall through for IDM_EDIT_CUT
                    If hbitmap
                        DeleteObject_(hbitmap)
                        hbitmap = #NULL
                    EndIf 
                    InvalidateRect_(hwin, #NULL, #TRUE)
                Case #IDM_EDIT_PASTE
                    If hbitmap
                        DeleteObject_(hbitmap)
                        hbitmap = #NULL
                    EndIf 
                    OpenClipboard_(hwin)
                    hbmp_clip = GetClipboardData_(#CF_BITMAP)
        
                    If hbmp_clip
                        hbitmap = CopyBitmap(hbmp_clip)
                    EndIf 
                    CloseClipboard_()
                    InvalidateRect_(hwin, #NULL, #TRUE)
                Default
                    ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
            EndSelect 
        Case #WM_PAINT
            Define ps.PAINTSTRUCT
            hdc = BeginPaint_(hwin, ps)

            If hbitmap
                Define rect.RECT, bm.BITMAP
                GetClientRect_(hwin, rect)
    
                hdc_mem = CreateCompatibleDC_(hdc)
                SelectObject_(hdc_mem, hbitmap)
                GetObject_(hbitmap, SizeOf(BITMAP), bm)
                SetStretchBltMode_(hdc, #COLORONCOLOR)
    
                StretchBlt_(hdc,    0, 0, rect\right, rect\bottom, hdc_mem, 0, 0, bm\bmWidth, bm\bmHeight, #SRCCOPY)
    
                DeleteDC_(hdc_mem)
            EndIf 

            EndPaint_(hwin, ps)
        Case #WM_DESTROY
            If hbitmap
                DeleteObject_(hbitmap)
            EndIf 
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect

    ProcedureReturn 0
EndProcedure
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
            \lpszMenuName = @app_name
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf
        
    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define haccel = LoadAccelerators_(hInstance, @app_name)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        If Not TranslateAccelerator_(hwin, haccel, msg)
            TranslateMessage_(msg)
            DispatchMessage_(msg)
        EndIf 
    Wend

End msg\wparam
