#IDM_FILE_OPEN = 40001

Macro CRLF:Chr(13) + Chr(10):EndMacro 
Macro TAB:chr(09):EndMacro

Procedure edit_print(hwin, text.s)
    SendMessage_(hwin, #EM_SETSEL, -1, -1)
    SendMessage_(hwin, #EM_REPLACESEL, #FALSE, text)
    SendMessage_(hwin, #EM_SCROLLCARET, 0, 0)
EndProcedure 

Dim info_name.s(3) 
Dim compression.s(4)
info_name(0) = "BITMAPCOREHEADER"
info_name(1) = "BITMAPINFOHEADER"
info_name(2) = "BITMAPV4HEADER"
info_name(3) = "BITMAPV5HEADER"

compression(0) = "BI_RGB"
compression(1) = "BI_RLE8"
compression(2) = "BI_RLE4"
compression(3) = "BI_BITFIELDS"
compression(4) = "unknown"

Procedure DisplayDibHeaders(hwin, file_name.s)
    Shared info_name(), compression()

    Define *pbmch.BITMAPCOREHEADER,  *pbmfh.BITMAPFILEHEADER, *pbmih.BITMAPV5HEADER
    Define bSuccess, file_size, high_size, bytes_read, hfile, i, *file, version.s

    ; Display the file name
    edit_print(hwin, "File: " + file_name + CRLF + CRLF)

    ; Open the file
    hfile = CreateFile_(@file_name, #GENERIC_READ, #FILE_SHARE_READ, #NULL, #OPEN_EXISTING, #FILE_FLAG_SEQUENTIAL_SCAN, #NULL)

    If hfile = #INVALID_HANDLE_VALUE
        edit_print(hwin, "Cannot open file." + CRLF + CRLF)
        ProcedureReturn 
    EndIf 
        
    ; Get the size of the file
    file_size = GetFileSize_(hfile, @high_size)

    If high_size
        edit_print(hwin, "Cannot deal with >4G files\" + CRLF + CRLF)
        CloseHandle_(hfile)
        ProcedureReturn 
    EndIf 
    
    ; Allocate memory for the file
    *file = AllocateMemory(file_size)

    If Not *file
        edit_print(hwin, "Cannot allocate memory\" + CRLF + CRLF)
        CloseHandle_(hfile)
        ProcedureReturn 
    EndIf 

    ; Read the file
    SetCursor_(LoadCursor_(#NULL, #IDC_WAIT))
    ShowCursor_(#TRUE)

    bSuccess = ReadFile_(hfile, *file, file_size, @bytes_read, #NULL)

    ShowCursor_(#FALSE)
    SetCursor_(LoadCursor_(#NULL, #IDC_ARROW))

    If (Not bSuccess) Or (bytes_read <> file_size)
        edit_print(hwin, "Could not read file." + CRLF + CRLF)
        CloseHandle_(hfile)
        FreeMemory(*file)
        ProcedureReturn 
    EndIf 

    ; Close the file
    CloseHandle_(hfile)

    ; Display file size
    edit_print(hwin, "File size = " + StrU(file_size) + " bytes" + CRLF + CRLF)

    ; Display BITMAPFILEHEADER structure
    *pbmfh = *file

    edit_print(hwin, "BITMAPFILEHEADER" + CRLF)
    edit_print(hwin, TAB + ".bfType = $" + Hex(*pbmfh\bfType) + CRLF)
    edit_print(hwin, TAB + ".bfSize = " + StrU(*pbmfh\bfSize) + CRLF)
    edit_print(hwin, TAB + ".bfReserved1 = " + StrU(*pbmfh\bfReserved1) + CRLF)
    edit_print(hwin, TAB + ".bfReserved2 = " + StrU(*pbmfh\bfReserved2) + CRLF)
    edit_print(hwin, TAB + ".bfOffBits = " + StrU(*pbmfh\bfOffBits) + CRLF + CRLF)

    ; Determine which information structure we have
    *pbmih = *file + SizeOf(BITMAPFILEHEADER)

    Select (*pbmih\bV5Size)
        Case SizeOf(BITMAPCOREHEADER) :   i = 0                
        Case SizeOf(BITMAPINFOHEADER) :   i = 1 : version = "i"   
        Case SizeOf(BITMAPV4HEADER)   :   i = 2 : version = "V4"  
        Case SizeOf(BITMAPV5HEADER)   :   i = 3 : version = "V5"  
        Default
            edit_print(hwin, "Unknown header size of "+StrU(*pbmih\bV5Size)+"." + CRLF + CRLF)
            FreeMemory(*file)
            ProcedureReturn 
    EndSelect

    edit_print(hwin, info_name(i) + CRLF)

    ; Display the BITMAPCOREHEADER fields
    If *pbmih\bV5Size = SizeOf(BITMAPCOREHEADER)
        CopyMemory(*pbmih, *pbmch, *pbmih\bV5Size) 

        edit_print(hwin, TAB + ".bcSize = " + StrU(*pbmch\bcSize) + CRLF)
        edit_print(hwin, TAB + ".bcWidth = " + StrU(*pbmch\bcWidth) + CRLF )
        edit_print(hwin, TAB + ".bcHeight = " + StrU(*pbmch\bcHeight) + CRLF)
        edit_print(hwin, TAB + ".bcPlanes = " + StrU(*pbmch\bcPlanes) + CRLF)
        edit_print(hwin, TAB + ".bcBitCount = " + StrU(*pbmch\bcBitCount) + CRLF + CRLF )
        FreeMemory(*file)
        ProcedureReturn 
    EndIf 

    ; Display the BITMAPINFOHEADER fields
    edit_print(hwin, TAB + ".b"+version+"Size = "+StrU(*pbmih\bV5Size) + CRLF)
    edit_print(hwin, TAB + ".b"+version+"Width = "+Str(*pbmih\bV5Width)+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Height = "+Str(*pbmih\bV5Height)+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Planes = "+StrU(*pbmih\bV5Planes)+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"BitCount = "+StrU(*pbmih\bV5BitCount) + CRLF)
    edit_print(hwin, TAB + ".b"+version+"Compression = "+ compression(min(4, *pbmih\bV5Compression)) + CRLF)

    edit_print(hwin, TAB + ".b"+version+"SizeImage = "+StrU(*pbmih\bV5SizeImage) + CRLF)
    edit_print(hwin, TAB + ".b"+version+"XPelsPerMeter = "+Str(*pbmih\bV5XPelsPerMeter) + CRLF)
    edit_print(hwin, TAB + ".b"+version+"YPelsPerMeter = "+Str(*pbmih\bV5YPelsPerMeter) + CRLF)
    edit_print(hwin, TAB + ".b"+version+"ClrUsed = "+Str(*pbmih\bV5ClrUsed) + CRLF)
    edit_print(hwin, TAB + ".b"+version+"ClrImportant = "+Str(*pbmih\bV5ClrImportant)+ CRLF + CRLF)

    If *pbmih\bV5Size = SizeOf(BITMAPINFOHEADER)
        If *pbmih\bV5Compression = #BI_BITFIELDS
            edit_print(hwin, "Red Mask   = "+RSet(Hex(*pbmih\bV5RedMask),8,"0")+ CRLF)
            edit_print(hwin, "Green Mask = "+RSet(Hex(*pbmih\bV5GreenMask),8,"0")+ CRLF)
            edit_print(hwin, "Blue Mask  = "+RSet(Hex(*pbmih\bV5BlueMask),8,"0")+ CRLF + CRLF)
        EndIf 
        FreeMemory(*file)
        ProcedureReturn 
    EndIf 

    ; Display additional BITMAPV4HEADER fields
    edit_print(hwin, TAB + ".b"+version+"RedMask   = "+RSet(Hex(*pbmih\bV5RedMask),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"GreenMask = "+RSet(Hex(*pbmih\bV5GreenMask),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"BlueMask  = "+RSet(Hex(*pbmih\bV5BlueMask),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"AlphaMask = "+RSet(Hex(*pbmih\bV5AlphaMask),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"CSType = "+StrU(*pbmih\bV5CSType)+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Endpoints\ciexyzRed\ciexyzX   = "+RSet(Hex(*pbmih\bV5Endpoints\ciexyzRed\ciexyzX),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Endpoints\ciexyzRed\ciexyzY   = "+RSet(Hex(*pbmih\bV5Endpoints\ciexyzRed\ciexyzY),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Endpoints\ciexyzRed\ciexyzZ   = "+RSet(Hex(*pbmih\bV5Endpoints\ciexyzRed\ciexyzZ),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Endpoints\ciexyzGreen\ciexyzX = "+RSet(Hex(*pbmih\bV5Endpoints\ciexyzGreen\ciexyzX),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Endpoints\ciexyzGreen\ciexyzY = "+RSet(Hex(*pbmih\bV5Endpoints\ciexyzGreen\ciexyzY),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Endpoints\ciexyzGreen\ciexyzZ = "+RSet(Hex(*pbmih\bV5Endpoints\ciexyzGreen\ciexyzZ),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Endpoints\ciexyzBlue\ciexyzX  = "+RSet(Hex(*pbmih\bV5Endpoints\ciexyzBlue\ciexyzX),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Endpoints\ciexyzBlue\ciexyzY  = "+RSet(Hex(*pbmih\bV5Endpoints\ciexyzBlue\ciexyzY),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Endpoints\ciexyzBlue\ciexyzZ  = "+RSet(Hex(*pbmih\bV5Endpoints\ciexyzBlue\ciexyzZ),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"GammaRed   = "+RSet(Hex(*pbmih\bV5GammaRed),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"GammaGreen = "+RSet(Hex(*pbmih\bV5GammaGreen),8,"0")+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"GammaBlue  = "+RSet(Hex(*pbmih\bV5GammaBlue),8,"0")+ CRLF + CRLF)

    If *pbmih\bV5Size = SizeOf(BITMAPV4HEADER)
        FreeMemory(*file)
        ProcedureReturn 
    EndIf 

    ; Display additional BITMAPV5HEADER fields
    edit_print(hwin, TAB + ".b"+version+"Intent = "+StrU(*pbmih\bV5Intent)+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"ProfileData = "+StrU(*pbmih\bV5ProfileData)+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"ProfileSize = "+StrU(*pbmih\bV5ProfileSize)+ CRLF)
    edit_print(hwin, TAB + ".b"+version+"Reserved = "+StrU(*pbmih\bV5Reserved)+ CRLF + CRLF)

    FreeMemory(*file)
EndProcedure 
