;
; showdib1.pb
; based on showdib1.c by Charles Petzold, in PW5E
IncludePath "..\"
XIncludeFile "petzold.pbi"

IncludePath ""
XIncludeFile "showdib1.pbi"
XIncludeFile "dibfile.pbi"

Global app_name.s = "showdib1", main_caption.s = "Show DIB #1"
Global hinstance = GetModuleHandle_(#NULL)

Procedure main_callback(hwin, message, wparam, lparam)
    Static *bmfh.BITMAPFILEHEADER, *bmi.BITMAPINFO, *bits
    Static client_width, client_height, dib_width, dib_height
    Static file_name.s{#MAX_PATH}, title_name.s{#MAX_PATH}
    Define success

    Select message
        Case #WM_PAINT
            Define ps.PAINTSTRUCT
            Define hdc = BeginPaint_(hwin, ps)

            If *bmfh
                SetDIBitsToDevice_(hdc, 0, 0, dib_width, dib_height, 0, 0, 0, dib_height, *bits, *bmi, #DIB_RGB_COLORS)
            EndIf 

            EndPaint_(hwin, ps)
        Case #WM_CREATE
            DibFileInitialize(hwin)
        Case #WM_SIZE
            client_width = LOWORD(lParam)
            client_height = HIWORD(lParam)
        Case #WM_INITMENUPOPUP
            If *bmfh
                Define state = #MF_ENABLED
            Else
                state = #MF_GRAYED
            EndIf 
            EnableMenuItem_(wParam, #IDM_FILE_SAVE, state)
        Case #WM_COMMAND
            Define id = LOWORD(wParam)
            Select id
                Case #IDM_FILE_OPEN
                    ; Show the File Open dialog box
                    If Not DibFileOpenDlg(hwin, @file_name, @title_name)
                        ProcedureReturn 0
                    EndIf

                    ; If there's an existing DIB, free the memory
                    If *bmfh
                        FreeMemory(*bmfh)
                        *bmfh = #NULL
                    EndIf 

                    ; Load the entire DIB into memory
                    SetCursor_(LoadCursor_(#NULL, #IDC_WAIT))
                    ShowCursor_(#TRUE)
        
                    *bmfh = DibLoadImage(file_name)
        
                    ShowCursor_(#FALSE)
                    SetCursor_(LoadCursor_(#NULL, #IDC_ARROW))
        
                    ; Invalidate the client area for later update
                    InvalidateRect_(hwin, #NULL, #TRUE)
        
                    If *bmfh = #NULL
                        MessageBox_(hwin, "Cannot load DIB file", app_name, 0)
                        ProcedureReturn 0
                    EndIf

                    ; Get pointers to the info structure & the bits
                    *bmi = *bmfh + SizeOf(BITMAPFILEHEADER)
                    *bits = *bmfh + *bmfh\bfOffBits
                    ; Get the DIB width and height
                    If *bmi\bmiHeader\biSize = SizeOf(BITMAPCOREHEADER)
                        Define *bmi_cast.BITMAPCOREHEADER = *bmi
                        dib_width = *bmi_cast\bcWidth
                        dib_height = *bmi_cast\bcHeight
                    Else
                        dib_width = *bmi\bmiHeader\biWidth
                        dib_height = absi(*bmi\bmiHeader\biHeight)
                    EndIf 
                Case #IDM_FILE_SAVE
                    ; Show the File Save dialog box
                    If Not DibFileSaveDlg(hwin, @file_name, @title_name)
                        ProcedureReturn 0
                    EndIf 
                    ; Save the DIB to a file
                    SetCursor_(LoadCursor_(#NULL, #IDC_WAIT))
                    ShowCursor_(#TRUE)
        
                    success = DibSaveImage(file_name, *bmfh)
        
                    ShowCursor_(#FALSE)
                    SetCursor_(LoadCursor_(#NULL, #IDC_ARROW))

                    If Not success
                        MessageBox_(hwin, "Cannot save DIB file", app_name, 0)
                    EndIf 
                Default
                    ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
            EndSelect 
        Case #WM_DESTROY
            If *bmfh
                FreeMemory(*bmfh)
            EndIf 
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect

    ProcedureReturn 0
EndProcedure
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
            \lpszMenuName = @app_name
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf
        
    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define haccel = LoadAccelerators_(hInstance, @app_name)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        If Not TranslateAccelerator_(hwin, haccel, msg)
            TranslateMessage_(msg)
            DispatchMessage_(msg)
        EndIf 
    Wend

End msg\wparam
