;
; seqdisp.pb
; based on seqdisp.c by Charles Petzold, in PW5E
; SLOWED DOWN TO ILLUSTRATE THE EFFECT
IncludePath "..\"
XIncludeFile "petzold.pbi"

#IDM_FILE_OPEN = 40001

Global app_name.s = "seqdisp", main_caption.s = "DIB Sequential Display"
Global hinstance = GetModuleHandle_(#NULL)

Procedure main_callback(hwin, message, wparam, lparam)
    Static *pbmi.BITMAPINFO, *pbits, dib_width, dib_height, bit_count
    Static ofn.OPENFILENAME, file_name.s{#MAX_PATH}, title_name.s{#MAX_PATH}
    DataSection
        ; PureBasic's string handling puts in the null terminators we need
        custom_file_filters: 
        Data.s "Bitmap Files(*\BMP)" 
        Data.s "*.bmp"
        Data.s "All Files(*.*)"
        Data.s "*.*"
        Data.c 0      ; Filter list should end with an extra null terminator.
                      ; Note the data type: PureBasic will make it byte or word
                      ; depending on compilation mode.
    EndDataSection

    Define bmfh.BITMAPFILEHEADER, success, topdown, bytes_Read, hfile
    Define info_size, bits_size, row_length, hdc

    Select message
        Case #WM_CREATE
            ofn\lStructSize      = SizeOf(OPENFILENAME)
            ofn\hwndOwner        = hwin
            ofn\lpstrFilter      = ?custom_file_filters
            ofn\lpstrFile        = @file_name
            ofn\nMaxFile        = #MAX_PATH
            ofn\lpstrFileTitle    = @title_name
            ofn\nMaxFileTitle    = #MAX_PATH
            ofn\lpstrDefExt      = @"bmp"
    Case #WM_COMMAND
            Define hMenu = GetMenu_(hwin), id = LOWORD(wparam)  
                
            Select id
                Case #IDM_FILE_OPEN
                    ; Display File Open dialog
                    If Not GetOpenFileName_(ofn)
                        ProcedureReturn 0
                    EndIf 
                    ; Get rid of old DIB        
                    If *pbmi
                        FreeMemory(*pbmi)
                        *pbmi = #NULL
                    EndIf 

                    If *pbits
                        FreeMemory(*pbits)
                        *pbits = #NULL
                    EndIf 

                    ; Generate WM_PAINT message to erase background
                    InvalidateRect_(hwin, #NULL, #TRUE)
                        UpdateWindow_(hwin)

                    ; Open the file
                    hfile = CreateFile_(@file_name, #GENERIC_READ, #FILE_SHARE_READ, #NULL, #OPEN_EXISTING, #FILE_FLAG_SEQUENTIAL_SCAN, #NULL)

                    If hfile = #INVALID_HANDLE_VALUE
                        MessageBox_(hwin, "Cannot open file.", app_name, #MB_ICONWARNING | #MB_OK)
                        ProcedureReturn 0
                    EndIf 

                    ; Read in the BITMAPFILEHEADER
                    success = ReadFile_(hfile, bmfh, SizeOf(BITMAPFILEHEADER), @bytes_Read, #NULL)

                    If (Not success) Or (bytes_Read <> SizeOf(BITMAPFILEHEADER))
                        MessageBox_(hwin, "Cannot read file.", app_name, #MB_ICONWARNING | #MB_OK)
                        CloseHandle_(hfile)
                        ProcedureReturn 0
                    EndIf 
        
                    ; Check that it's a bitmap
                    Define bf_type.s = PeekS( @bmfh\bfType, 2, #PB_Ascii )

                    If bf_type <> "BM"
                        MessageBox_(hwin, "File is not a bitmap.", app_name, #MB_ICONWARNING | #MB_OK)
                        CloseHandle_(hfile)
                        ProcedureReturn 0
                    EndIf 
        
                    ; Allocate memory for header and bits
                    info_size = bmfh\bfOffBits - SizeOf(BITMAPFILEHEADER)
                    bits_size = bmfh\bfSize - bmfh\bfOffBits
        
                    *pbmi  = AllocateMemory(info_size)
                    *pbits = AllocateMemory(bits_size)
        
                    If *pbmi = #NULL Or *pbits = #NULL
                        MessageBox_(hwin, "Cannot allocate memory.", app_name, #MB_ICONWARNING | #MB_OK)
                        If *pbmi 
                            FreeMemory(*pbmi)
                        EndIf 
                        If *pbits 
                            FreeMemory(*pbits)
                        EndIf 
                        CloseHandle_(hfile)
                        ProcedureReturn 0
                    EndIf 
        
                    ; Read in the Information Header
                    success = ReadFile_(hfile, *pbmi, info_size, @bytes_Read, #NULL)
        
                    If (Not success) Or (bytes_Read <> info_size)
                        MessageBox_(hwin, "Cannot read file.", app_name, #MB_ICONWARNING | #MB_OK)
                        If *pbmi 
                            FreeMemory(*pbmi)
                        EndIf 
                        If *pbits 
                            FreeMemory(*pbits)
                        EndIf 
                        CloseHandle_(hfile)
                        ProcedureReturn 0
                    EndIf 
        
                    ; Get the DIB width and height
                    topdown = #FALSE

                    If *pbmi\bmiHeader\biSize = SizeOf(BITMAPCOREHEADER)
                        Define *pbch.BITMAPCOREHEADER = *pbmi  ; cast
                        dib_width = *pbch\bcWidth
                        dib_height =*pbch\bcHeight
                        bit_count = *pbch\bcBitCount
                    Else
                        If *pbmi\bmiHeader\biHeight < 0
                            topdown = #TRUE
                        EndIf 
                        
                        dib_width =     *pbmi\bmiHeader\biWidth
                        dib_height = absi(*pbmi\bmiHeader\biHeight)
                        bit_count =     *pbmi\bmiHeader\biBitCount
        
                        If (*pbmi\bmiHeader\biCompression <> #BI_RGB) And (*pbmi\bmiHeader\biCompression <> #BI_BITFIELDS)
                            MessageBox_(hwin, "File is compressed.", app_name, #MB_ICONWARNING | #MB_OK)
                            If *pbmi 
                                FreeMemory(*pbmi)
                            EndIf 
                            If *pbits 
                                FreeMemory(*pbits)
                            EndIf 
                            CloseHandle_(hfile)
                            ProcedureReturn 0
                        EndIf 
                    EndIf 

                    ; Get the row length
                    row_length = ((dib_width * bit_count + 31) & (~31) ) >> 3
        
                    ; Read and display
                    SetCursor_(LoadCursor_(#NULL, #IDC_WAIT))
                    ShowCursor_(#TRUE)
        
                    hdc = GetDC_(hwin)
                    Define y
                    For y = 0 To dib_height-1 
                        ReadFile_(hfile, *pbits + y *row_length, row_length, @bytes_Read, #NULL)
                        Define first_scan
                        If topdown
                            first_scan = dib_height - y - 1 
                            ; slow it down for illustration
                            If y < 500
                                Delay(10)
                            EndIf 
                        Else 
                            first_scan = y
                            ; slow it down for illustration
                            If (dib_height - y < 500)
                                Delay(10)
                            EndIf 
                        EndIf   
                        SetDIBitsToDevice_(hdc, 0, 0, dib_width, dib_height, 0, 0, first_scan, 1, *pbits + y *row_length, *pbmi, #DIB_RGB_COLORS)

                    Next y
                ReleaseDC_(hwin, hdc)
                CloseHandle_(hfile)
                ShowCursor_(#FALSE)
                SetCursor_(LoadCursor_(#NULL, #IDC_ARROW))
                ProcedureReturn 0
            EndSelect 
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
        Case #WM_PAINT
            Define ps.PAINTSTRUCT
            hdc = BeginPaint_(hwin, ps)

            If *pbmi And *pbits
                SetDIBitsToDevice_(hdc, 0, 0, dib_width, dib_height, 0, 0, 0, dib_height, *pbits, *pbmi, #DIB_RGB_COLORS)
            EndIf 
            EndPaint_(hwin, ps)
        Case #WM_DESTROY
            If *pbmi
                FreeMemory(*pbmi)
            EndIf 
            If *pbits
                FreeMemory(*pbits)
            EndIf 
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect

    ProcedureReturn 0
EndProcedure
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
            \lpszMenuName = @app_name
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf
        
    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define haccel = LoadAccelerators_(hInstance, app_name)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        If Not TranslateAccelerator_(hwin, haccel, msg)
            TranslateMessage_(msg)
            DispatchMessage_(msg)
        EndIf
    Wend

End msg\wparam
