; DIB file functions

Define ofn.OPENFILENAME
    
Procedure DibFileInitialize(hwin)
    Shared ofn
    DataSection
        ; PureBasic's string handling puts in the null terminators we need
        custom_file_filters: 
        Data.s "Bitmap Files(*\BMP)" 
        Data.s "*.bmp"
        Data.s "All Files(*.*)"
        Data.s "*.*"
        Data.c 0      ; Filter list should end with an extra null terminator.
                      ; Note the data type: PureBasic will make it byte or word
                      ; depending on compilation mode.
    EndDataSection

    ofn\lStructSize      = SizeOf(OPENFILENAME)
    ofn\hwndOwner        = hwin
    ofn\lpstrFilter      = ?custom_file_filters
    ofn\nMaxFile        = #MAX_PATH
    ofn\nMaxFileTitle    = #MAX_PATH
    ofn\lpstrDefExt      = @"bmp"
EndProcedure 

Procedure DibFileOpenDlg(hwin, *file_name, *title_name)
    Shared ofn

    ofn\hwndOwner        = hwin
    ofn\lpstrFile        = *file_name
    ofn\lpstrFileTitle   = *title_name
    ofn\Flags            = 0

    ProcedureReturn GetOpenFileName_(ofn)  
EndProcedure 

Procedure DibFileSaveDlg(hwin, *file_name, *title_name)
    Shared ofn
    ofn\hwndOwner        = hwin
    ofn\lpstrFile        = *file_name
    ofn\lpstrFileTitle    = *title_name
    ofn\Flags           = #OFN_OVERWRITEPROMPT

    ProcedureReturn GetSaveFileName_(ofn)
EndProcedure 

Procedure DibLoadImage(file_name.s)
    Shared ofn
    Define success, file_size, high_size, bytes_read
    Define hfile, *pbmfh.BITMAPFILEHEADER

    hfile = CreateFile_(file_name, #GENERIC_READ, #FILE_SHARE_READ, #NULL, #OPEN_EXISTING, #FILE_FLAG_SEQUENTIAL_SCAN, #NULL)

    If hfile = #INVALID_HANDLE_VALUE
        ProcedureReturn #NULL
    EndIf 
    file_size = GetFileSize_(hfile, @high_size)

    If high_size ; file > 4Gb
        CloseHandle_(hfile)
        ProcedureReturn #NULL
    EndIf 

    *pbmfh = AllocateMemory(file_size)

    If Not *pbmfh
        CloseHandle_(hfile)
        ProcedureReturn #NULL
    EndIf 

    success = ReadFile_(hfile, *pbmfh, file_size, @bytes_read, #NULL)
    CloseHandle_(hfile)
    
    Define bf_type.s = PeekS( @*pbmfh\bfType, 2, #PB_Ascii )
    If (Not success) Or (bytes_read <> file_size) Or (bf_type <> "BM") Or (*pbmfh\bfSize <> file_size)
        FreeMemory(*pbmfh)
        ProcedureReturn #NULL
    EndIf 
    ProcedureReturn *pbmfh
EndProcedure 

Procedure DibSaveImage(file_name.s, *pbmfh.BITMAPFILEHEADER)
    Shared ofn
    Define success, bytes_written, hfile

    hfile = CreateFile_(file_name, #GENERIC_WRITE, 0, #NULL, #CREATE_ALWAYS, #FILE_ATTRIBUTE_NORMAL, #NULL)

    If hfile = #INVALID_HANDLE_VALUE
        ProcedureReturn #FALSE
    EndIf
    success = WriteFile_(hfile, *pbmfh, *pbmfh\bfSize, @bytes_written, #NULL)
    CloseHandle_(hfile)

    If (Not success) Or (bytes_written <> *pbmfh\bfSize)
        DeleteFile_(file_name)
        ProcedureReturn #FALSE
    EndIf 
    ProcedureReturn #TRUE
EndProcedure 
    