;
; showdib2.pb
; based on showdib2.c by Charles Petzold, in PW5E
IncludePath "..\"
XIncludeFile "petzold.pbi"

IncludePath ""
XIncludeFile "dibfile.pbi"
XIncludeFile "showdib2.pbi"

Global app_name.s = "showdib2", main_caption.s = "Show DIB #2"
Global hinstance = GetModuleHandle_(#NULL)

Procedure main_callback(hwin, message, wparam, lparam)
    Static *bmfh.BITMAPFILEHEADER, *bmi.BITMAPINFO, *bits
    Static client_width, client_height, dib_width, dib_height
    Static file_name.s{#MAX_PATH}, title_name.s{#MAX_PATH}
    Static di.DOCINFO, printdlg.PRINTDLG
    Static show = #IDM_SHOW_NORMAL
    Define hglobal, hmenu, page_width, page_height, menu_state, *globalmem, success

    Select message
        Case #WM_PAINT
            Define ps.PAINTSTRUCT
            Define hdc = BeginPaint_(hwin, ps)

            If *bmfh
                show_dib(hdc, *bmi, *bits, dib_width, dib_height, client_width, client_height, show)
            EndIf 

            EndPaint_(hwin, ps)
        Case #WM_CREATE
            di\cbsize = SizeOf(DOCINFO)
            di\lpszDocName = @"ShowDib2: Printing"
            printdlg\lStructSize = SizeOf(PRINTDLG)

            DibFileInitialize(hwin)
        Case #WM_SIZE
            client_width = LOWORD(lParam)
            client_height = HIWORD(lParam)
        Case #WM_INITMENUPOPUP
            hmenu = GetMenu_(hwin)
    
            If *bmfh
                menu_state = #MF_ENABLED
            Else
                menu_state = #MF_GRAYED
            EndIf 
            EnableMenuItem_(hmenu, #IDM_FILE_SAVE,   menu_state)
            EnableMenuItem_(hmenu, #IDM_FILE_PRINT,  menu_state)
            EnableMenuItem_(hmenu, #IDM_EDIT_CUT,    menu_state)
            EnableMenuItem_(hmenu, #IDM_EDIT_COPY,   menu_state)
            EnableMenuItem_(hmenu, #IDM_EDIT_DELETE, menu_state)
        Case #WM_COMMAND
            hmenu = GetMenu_(hwin)

            Define id = LOWORD(wParam)
            Select id
                Case #IDM_FILE_OPEN
                    ; Show the File Open dialog box
                    If Not DibFileOpenDlg(hwin, @file_name, @title_name)
                        ProcedureReturn 0
                    EndIf

                    ; If there's an existing DIB, free the memory
                    If *bmfh
                        FreeMemory(*bmfh)
                        *bmfh = #NULL
                    EndIf 

                    ; Load the entire DIB into memory
                    SetCursor_(LoadCursor_(#NULL, #IDC_WAIT))
                    ShowCursor_(#TRUE)
        
                    *bmfh = DibLoadImage(file_name)
        
                    ShowCursor_(#FALSE)
                    SetCursor_(LoadCursor_(#NULL, #IDC_ARROW))
        
                    ; Invalidate the client area for later update
                    InvalidateRect_(hwin, #NULL, #TRUE)
        
                    If *bmfh = #NULL
                        MessageBox_(hwin, "Cannot load DIB file", app_name, #MB_ICONEXCLAMATION | #MB_OK)
                        ProcedureReturn 0
                    EndIf

                    ; Get pointers to the info structure & the bits
                    *bmi = *bmfh + SizeOf(BITMAPFILEHEADER)
                    *bits = *bmfh + *bmfh\bfOffBits
                    ; Get the DIB width and height
                    If *bmi\bmiHeader\biSize = SizeOf(BITMAPCOREHEADER)
                        Define *bmi_cast.BITMAPCOREHEADER = *bmi
                        dib_width = *bmi_cast\bcWidth
                        dib_height = *bmi_cast\bcHeight
                    Else
                        dib_width = *bmi\bmiHeader\biWidth
                        dib_height = absi(*bmi\bmiHeader\biHeight)
                    EndIf 
                Case #IDM_FILE_SAVE
                    ; Show the File Save dialog box
                    If Not DibFileSaveDlg(hwin, @file_name, @title_name)
                        ProcedureReturn 0
                    EndIf 
                    ; Save the DIB to a disk file
                    SetCursor_(LoadCursor_(#NULL, #IDC_WAIT))
                    ShowCursor_(#TRUE)
        
                    success = DibSaveImage(file_name, *bmfh)
        
                    ShowCursor_(#FALSE)
                    SetCursor_(LoadCursor_(#NULL, #IDC_ARROW))

                    If Not success
                        MessageBox_(hwin, "Cannot save DIB file", app_name, 0)
                    EndIf 
                Case #IDM_FILE_PRINT
                    If Not *bmfh
                        ProcedureReturn 0
                    EndIf 
                    
                    ; Get printer DC
                	printdlg\Flags = #PD_RETURNDC | #PD_NOPAGENUMS | #PD_NOSELECTION

        	        If Not PrintDlg_(printdlg)
                        ProcedureReturn 0
                    EndIf 

                    Define hdc_prn = printdlg\hDC
                    If hdc_prn = #NULL 
                        MessageBox_(hwin, "Cannot obtain Printer DC", app_name, #MB_ICONEXCLAMATION | #MB_OK)
                        ProcedureReturn 0
                    EndIf 

                    ; Check If the printer can print bitmaps        
                    If Not (#RC_BITBLT & GetDeviceCaps_(hdc_prn, #RASTERCAPS))
                        DeleteDC_(hdc_prn)
                        MessageBox_(hwin, "Printer cannot print bitmaps", app_name, #MB_ICONEXCLAMATION | #MB_OK)
                        ProcedureReturn 0
                    EndIf 
                    
                    ; Get size of printable area of page        
                    page_width = GetDeviceCaps_(hdc_prn, #HORZRES)
                    page_height = GetDeviceCaps_(hdc_prn, #VERTRES)
        
                    success = #FALSE
        
                    ; Send the DIB to the printer        
                    SetCursor_(LoadCursor_(#NULL, #IDC_WAIT))
                    ShowCursor_(#TRUE)
        
                    If (StartDoc_(hdc_prn, di) > 0) And (StartPage_(hdc_prn) > 0)
                        show_dib(hdc_prn, *bmi, *bits, dib_width, dib_height, page_width, page_height, show)
                        
                        If EndPage_(hdc_prn) > 0
                            success = #TRUE
                            EndDoc_(hdc_prn)
                        EndIf 
                    EndIf 
                    ShowCursor_(#FALSE)
                    SetCursor_(LoadCursor_(#NULL, #IDC_ARROW))
        
                    DeleteDC_(hdc_prn)
        
                    If Not success
                        MessageBox_(hwin, "Could not print bitmap", app_name, #MB_ICONEXCLAMATION | #MB_OK)
                    EndIf         
                Case #IDM_EDIT_COPY, #IDM_EDIT_CUT, #IDM_EDIT_DELETE
                    If id <> #IDM_EDIT_DELETE
                        If Not *bmfh
                            ProcedureReturn 0
                        EndIf 
            
                        ; Make a copy of the packed DIB        
                        hglobal = GlobalAlloc_(#GHND | #GMEM_SHARE, *bmfh\bfSize - SizeOf(BITMAPFILEHEADER))
            
                        *globalmem = GlobalLock_(hglobal)
            
                        CopyMemory_(*globalmem, *bmfh + SizeOf(BITMAPFILEHEADER), *bmfh\bfSize - SizeOf(BITMAPFILEHEADER))
            
                        GlobalUnlock_(hglobal)
            
                        ; Transfer it to the clipboard        
                        OpenClipboard_(hwin)
                        EmptyClipboard_()
                        SetClipboardData_(#CF_DIB, hglobal)
                        CloseClipboard_()
            
                        If id = #IDM_EDIT_COPY ;;;;; fall through If #IDM_EDIT_CUT
                            ProcedureReturn 0
                        EndIf 
                    EndIf 
                    If id <> #IDM_EDIT_COPY 
                        If *bmfh
                            FreeMemory(*bmfh)
                            *bmfh = #NULL
                            InvalidateRect_(hwin, #NULL, #TRUE)
                        EndIf 
                    EndIf 
                Case #IDM_SHOW_NORMAL, #IDM_SHOW_CENTER, #IDM_SHOW_STRETCH, #IDM_SHOW_ISOSTRETCH
                    CheckMenuItem_(hmenu, show, #MF_UNCHECKED)
                    
                    show = id
                    
                    CheckMenuItem_(hmenu, show, #MF_CHECKED)
                    InvalidateRect_(hwin, #NULL, #TRUE)
                Default
                    ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
            EndSelect 
        Case #WM_DESTROY
            If *bmfh
                FreeMemory(*bmfh)
            EndIf 
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect

    ProcedureReturn 0
EndProcedure
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
            \lpszMenuName = @app_name
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf
        
    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define haccel = LoadAccelerators_(hinstance, app_name)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        If Not TranslateAccelerator_(hwin, haccel, msg)
            TranslateMessage_(msg)
            DispatchMessage_(msg)
        EndIf 
    Wend

End msg\wparam
