;
; dibsect.pb
; based on dibsect.c by Charles Petzold, in PW5E
IncludePath "..\"
XIncludeFile "petzold.pbi"

IncludePath ""
XIncludeFile "dibsect.pbi"

Global app_name.s = "dibsect", main_caption.s = "DIB Section Display"
Global hinstance = GetModuleHandle_(#NULL)

#IDM_FILE_OPEN = 40001

Procedure main_callback(hwin, message, wparam, lparam)
    Static hbitmap, client_width, client_height, ofn.OPENFILENAME
    Static file_name.s{#MAX_PATH}, title_name.s{#MAX_PATH}
    DataSection
        ; PureBasic's string handling puts in the null terminators we need
        custom_file_filters: 
        Data.s "Bitmap Files(*\BMP)" 
        Data.s "*.bmp"
        Data.s "All Files(*.*)"
        Data.s "*.*"
        Data.c 0      ; Filter list should end with an extra null terminator.
                      ; Note the data type: PureBasic will make it byte or word
                      ; depending on compilation mode.
    EndDataSection
    Define bitmap.BITMAP, hdc

    Select message
        Case #WM_CREATE
            ofn\lStructSize      = SizeOf(OPENFILENAME)
            ofn\hwndOwner        = hwin
            ofn\lpstrFilter      = ?custom_file_filters
            ofn\lpstrFile        = @file_name
            ofn\nMaxFile        = #MAX_PATH
            ofn\lpstrFileTitle    = @title_name
            ofn\nMaxFileTitle    = #MAX_PATH
            ofn\lpstrDefExt      = @"bmp"
        Case #WM_SIZE
            client_width = LOWORD(lParam)
            client_height = HIWORD(lParam)
        Case #WM_PAINT
            Define ps.PAINTSTRUCT
            hdc = BeginPaint_(hwin, ps)
    
            If hbitmap
                GetObject_(hbitmap, SizeOf(BITMAP), bitmap)
    
                Define hdc_mem = CreateCompatibleDC_(hdc)
                SelectObject_(hdc_mem, hbitmap)

                BitBlt_(hdc,    0, 0, bitmap\bmWidth, bitmap\bmHeight, hdc_mem, 0, 0, #SRCCOPY)
    
                DeleteDC_(hdc_mem)
            EndIf 
    
            EndPaint_(hwin, ps)
        Case #WM_COMMAND
            Define id = LOWORD(wparam)
            Select id
                Case #IDM_FILE_OPEN
                    ; Show the File Open dialog box
                    If Not GetOpenFileName_(ofn)
                        ProcedureReturn 0
                    EndIf
                    
                    ; If there's an existing DIB, delete it
                    If hbitmap
                        DeleteObject_(hbitmap)
                        hbitmap = #NULL
                    EndIf

                    ; Create the DIB Section from the DIB file
                    SetCursor_(LoadCursor_(#NULL, #IDC_WAIT))
                    ShowCursor_(#TRUE)
        
                    hbitmap = CreateDibSectionFromDibFile(file_name)
        
                    ShowCursor_(#FALSE)
                    SetCursor_(LoadCursor_(#NULL, #IDC_ARROW))

                    ; Invalidate the client area for later update
                    InvalidateRect_(hwin, #NULL, #TRUE)
        
                    If hbitmap = #NULL
                        MessageBox_(hwin, "Cannot load DIB file", app_name, #MB_OK | #MB_ICONEXCLAMATION)
                    EndIf
                    ProcedureReturn 0
            EndSelect
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)    

        Case #WM_DESTROY
            If hbitmap
                DeleteObject_(hbitmap)
            EndIf 
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect

    ProcedureReturn 0
EndProcedure
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
            \lpszMenuName = @app_name
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf
        
    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        TranslateMessage_(msg)
        DispatchMessage_(msg)
    Wend

End msg\wparam