;
; grays3.pb
; based on grays3.c by Charles Petzold, in PW5E
IncludePath "..\"
XIncludeFile "petzold.pbi"

Global app_name.s = "grays3", main_caption.s = "Shades of Gray #3"
Global hinstance = GetModuleHandle_(#NULL)

Procedure main_callback(hwin, message, wparam, lparam)
    Static hpalette 

    Select message
        Case #WM_SIZE
            Static client_width, client_height
            client_width = LOWORD(lparam)
            client_height = HIWORD(lparam)
        Case #WM_PAINT
            Define ps.PAINTSTRUCT
            Define hdc = BeginPaint_(hwin, ps)

            ; Select and realize the palette in the device context
            SelectPalette_(hdc, hpalette, #FALSE)
            RealizePalette_(hdc)

            ; Draw the fountain of grays
            Define i, rect.RECT, hbrush
            For i = 0 To 64
                rect\left   = i * client_width / 64
                rect\top    = 0
                rect\right  = (i + 1) * client_width / 64
                rect\bottom = client_height
    
                hBrush = CreateSolidBrush_(PALETTEINDEX(i))
                
                FillRect_(hdc, rect, hbrush)
                DeleteObject_(hbrush)
            Next i

            EndPaint_(hwin, ps)
        Case #WM_QUERYNEWPALETTE
            If Not hpalette
                ProcedureReturn #FALSE
            EndIf 
            
            hdc = GetDC_(hwin)
            SelectPalette_(hdc, hpalette, #FALSE)
            RealizePalette_(hdc)
            InvalidateRect_(hwin, #NULL, #TRUE)
    
            ReleaseDC_(hwin, hdc)
            ProcedureReturn #TRUE
    
        Case #WM_PALETTECHANGED
            If (Not hpalette) Or (wparam = hwin)
                ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
            EndIf 
            
            hdc = GetDC_(hwin)
            SelectPalette_(hdc, hpalette, #FALSE)
            RealizePalette_(hdc)
            UpdateColors_(hdc)
    
            ReleaseDC_(hwin, hdc)
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)    
        Case #WM_CREATE
            ; Set up a LOGPALETTE structure and create a palette
            Define *plp.LOGPALETTE = AllocateMemory(SizeOf(LOGPALETTE) + 64 *SizeOf(PALETTEENTRY))

            *plp\palVersion    = $0300
            *plp\palNumEntries = 65

            ; Notice that this is being tricky with the last member of the
            ; logpalette struct; it is defined as only a one element array but
            ; PureBasic is letting us get away with referencing it as however
            ; many indexes we want (i.e., like C).
            For i = 0 To 64
                *plp\palPalEntry[i]\peRed   = min(255, 4 * i)
                *plp\palPalEntry[i]\peGreen = min(255, 4 * i)
                *plp\palPalEntry[i]\peBlue  = min(255, 4 * i)
                *plp\palPalEntry[i]\peFlags = 0
            Next i
            hpalette = CreatePalette_(*plp)
            FreeMemory(*plp)
        Case #WM_DESTROY
            DeleteObject_(hpalette)
            PostQuitMessage_(0)
        Default
            ProcedureReturn DefWindowProc_(hwin, message, wparam, lparam)
    EndSelect

    ProcedureReturn 0
EndProcedure
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN PROGRAM CODE
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    Define main_class.WNDCLASS
        With main_class
            \lpfnWndProc = @main_callback()
            \hinstance = hinstance
            \lpszClassName = @app_name
            \style = #CS_HREDRAW | #CS_VREDRAW
            \hicon = LoadIcon_(0, #IDI_APPLICATION)
            \hcursor = LoadCursor_(0, #IDC_ARROW)
            \hbrBackground = #COLOR_WINDOW + 1
        EndWith
        If Not RegisterClass_(main_class)
            MessageBox_(#NULL, "Cannot register window class!", app_name, #MB_ICONERROR)
            End 0
        EndIf

    Define hwin = CreateWindowEx_(#NULL,app_name,main_caption,#WS_OVERLAPPEDWINDOW | #WS_CLIPCHILDREN,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#CW_USEDEFAULT,#NULL, #NULL,hinstance,#NULL)
        If Not hwin
            MessageBox_(#NULL, "Unable to create main window.", app_name, #MB_ICONERROR)
            End 0
        EndIf

        ShowWindow_(hwin, #SW_SHOW)
        UpdateWindow_(hwin)

    Define msg.MSG
    While GetMessage_(msg, #NULL, 0, 0) > 0
        TranslateMessage_(msg)
        DispatchMessage_(msg)
    Wend

End msg\wparam
