EnableExplicit
;
; petzold.pbi
;
; by maker
; http://programmingforfun.net
;
; =========================================
; PROCESSOR DEPENDENT STUFF (pointer size constant, etc)
; =========================================
CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
    #pb_pointer_size = 4
CompilerElse
    CompilerIf #PB_Compiler_Processor = #PB_Processor_x64
        #pb_pointer_size = 8
    CompilerEndIf
CompilerEndIf

; =========================================
;; Type definitions for SizeOf() convenience
; =========================================
Structure CHAR    :   var.c   : EndStructure

; =========================================
; API MACROS; 
; =========================================
Macro HIWORD(n) :  (n >> 16) : EndMacro
Macro LOWORD(n) : (n & $FFFF) : EndMacro
Macro MAKEINTRESOURCE(n) :  LOWORD(n) :  EndMacro
Macro MAKELONG(low,high) : (((low) & $FFFF) | (((high) & $FFFF) << 16)) : EndMacro
Macro PALETTEINDEX(i)    : ($01000000 | ($FFFF & i))    : EndMacro  
Macro PALETTERGB(r, g, b): ($02000000 | RGB(r, g, b))   : EndMacro

Macro DialogBox_(hModule, pTemplateName, hOwner, pDialogFunc) :DialogBoxParam_(hModule, pTemplateName, hOwner, pDialogFunc, #Null): EndMacro
Macro CreateDialog_(hModule, pTemplateName, hOwner, pDialogFunc):CreateDialogParam_(hModule, pTemplateName, hOwner, pDialogFunc, #Null):EndMacro
; =========================================
; CORRECTIONS FOR API STRUCTURE MISTAKES
;
Macro lpSetupTemplateName: pPrintSetupTemplateName :EndMacro  ; PB 4.3 has a mistake in the PRINTDLG structure definition
Macro ciexyzBlue: ciexyBlue  :EndMacro ; PB 4.3 has this type in the CIEXYZTRIPLE structure definition 

 
; =========================================
; MISSING WINDOWS CONSTANTS
;
#GCLP_HBRBACKGROUND = -10
#IS_TEXT_UNICODE_REVERSE_SIGNATURE = $80
#IS_TEXT_UNICODE_SIGNATURE = 8
#WHEEL_DELTA = 120

Macro FINDMSGSTRING:"commdlg_FindReplace":EndMacro
; =========================================
; MISSING WINDOWS API DECLARATIONS

; =========================================;
; WINDOW CLASS CONSTANTS
;
Macro WC_BUTTON         :"Button"           :EndMacro
Macro WC_COMBOBOX       :"ComboBox"         :EndMacro
Macro WC_COMBOBOXEX     :"ComboBoxEx32"     :EndMacro
Macro WC_EDIT           :"Edit"             :EndMacro
Macro WC_HEADER         :"SysHeader32"      :EndMacro
Macro WC_IPADDRESS      :"SysIPAddress32"   :EndMacro
Macro WC_LINK           :"SysLink"          :EndMacro
Macro WC_LISTBOX        :"ListBox"          :EndMacro
Macro WC_LISTVIEW       :"SysListView32"    :EndMacro
Macro WC_PAGESCROLLER   :"SysPager"         :EndMacro
Macro WC_SCROLLBAR      :"ScrollBar"        :EndMacro
Macro WC_STATIC         :"Static"           :EndMacro
Macro WC_TABCONTROL     :"SysTabControl32"  :EndMacro
Macro WC_TREEVIEW       :"SysTreeView32"    :EndMacro
CompilerIf #PB_Compiler_Unicode = 1
    Macro WC_RICHED20       :"richedit20w"       :EndMacro
CompilerElse 
    Macro WC_RICHED20       :"richedit20a"       :EndMacro
CompilerEndIf
; =========================================;

; =========================================;
;  MY MACROS
; =========================================;
Macro line_to(hdc, x, y, end_x, end_y, pprev_pos_buffer=#Null)
    MoveToEx_(hdc, x, y, pprev_pos_buffer)
    LineTo_(hdc, end_x, end_y)
EndMacro

Macro text_out(hdc,x,y,pstr)
    TextOut_(hdc,x,y,pstr,Len(pstr))
EndMacro

; =========================================;
; MY ROUTINES
; =========================================;
Procedure.i max(a.q, b.q)
    If a > b
        ProcedureReturn a
    Else
        ProcedureReturn b
    EndIf
EndProcedure

Procedure.i min(a.q, b.q)
    If a < b
        ProcedureReturn a
    Else
        ProcedureReturn b
    EndIf
EndProcedure

Procedure.i file_exists(filePath.s)
    Define size.q
    size = FileSize(filePath)
    If size = -1 Or size = -2
        ProcedureReturn #False
    Else
        ProcedureReturn #True
    EndIf
EndProcedure

Procedure subfolder_exists(directory.s)
; look in current folder for specified subfolder
; Return: 1  subfolder exists
;         0  subfolder does not exist
;        -1  error, unable to examine directory system
    Define dir_num = ExamineDirectory(#PB_Any, GetCurrentDirectory(), "*.*")
    If dir_num
        While NextDirectoryEntry(dir_num)
            If DirectoryEntryType(dir_num) = #PB_DirectoryEntry_Directory
                If directory = DirectoryEntryName(dir_num)
                    ProcedureReturn 1
                EndIf
            EndIf
        Wend
        FinishDirectory(dir_num)
        ProcedureReturn 0
    Else
        ProcedureReturn -1
    EndIf
EndProcedure


Procedure absi(num)
; integer absolute value
    If num < 0 
        ProcedureReturn -num
    Else
        ProcedureReturn num
    EndIf 

EndProcedure 

Procedure set_control_font(hcontrol, *lf.LOGFONT)
    Define holdfont = SendMessage_(hcontrol, #WM_GETFONT, 0, 0)

    Define hfont = CreateFontIndirect_(*lf)
    SendMessage_(hcontrol, #WM_SETFONT, hfont, #True)

    If holdfont <> 0 
        If DeleteObject_(holdfont) = 0 
            ProcedureReturn #False 
        EndIf 
    EndIf 
    ProcedureReturn #True 
EndProcedure 


; =========================================
; Get the Default printer DC 
; =========================================
#DEFAULTPRINTERDC_BUFFERSIZE = 500
Procedure default_printer_dc()
  ; Returns default printer dc or zero on failure. 
  ;converted from some MSDN code.
    Define hdc
    Define ver.OSVERSIONINFO
    
    ver\dwOSVersionInfoSize = SizeOf(OSVERSIONINFO)
    If Not GetVersionEx_(ver)
        ProcedureReturn #Null
    EndIf

    If ver\dwPlatformId = #VER_PLATFORM_WIN32_WINDOWS ; Windows 95/98
        Define cb_needed, cb_returned, *info2.PRINTER_INFO_5
        EnumPrinters_(#PRINTER_ENUM_DEFAULT, #Null, 2, #Null, 0, @cb_needed, @cb_returned)

        If (GetLastError_() <> #ERROR_INSUFFICIENT_BUFFER) Or (cb_needed = 0)
            ProcedureReturn #Null
        EndIf

        *info2 = AllocateMemory(cb_needed)

        If Not EnumPrinters_(#PRINTER_ENUM_DEFAULT, #Null, 2, *info2, @cb_needed, @cb_needed, @cb_returned)
            FreeMemory(*info2)
            ProcedureReturn #Null
        EndIf
        
        hdc = CreateDC_(#Null, *info2\pPrinterName, #Null, #Null)

        FreeMemory(*info2)
    ElseIf ver\dwMajorVersion >= 5   ; Windows 2000 or higher
        Define hwin_spool = OpenLibrary(#PB_Any, "winspool.drv")
        If Not hwin_spool
            ProcedureReturn #Null
        EndIf

        CompilerIf #PB_Compiler_Unicode = 1
            Macro GETDEFAULTPRINTER : "GetDefaultPrinterW" : EndMacro
        CompilerElse
            Macro GETDEFAULTPRINTER : "GetDefaultPrinterA" : EndMacro
        CompilerEndIf

        Define *get_def_prn = GetFunction(hwin_spool, GETDEFAULTPRINTER)

        If Not *get_def_prn
            CloseLibrary(hwin_spool)
            ProcedureReturn #Null
        EndIf

        Define cb_buffer
        If CallFunctionFast(*get_def_prn,#Null, @cb_buffer)  ; no printer to get if successful with null buffer pointer
            CloseLibrary(hwin_spool)
            ProcedureReturn #Null
        EndIf
        
        Define *prn_name = AllocateMemory(cb_buffer)
        If Not CallFunctionFast(*get_def_prn, *prn_name, @cb_buffer)
            CloseLibrary(hwin_spool)
            ProcedureReturn #Null
        EndIf
        CloseLibrary(hwin_spool)
        
        hdc = CreateDC_(#Null, *prn_name, #Null, #Null)
        FreeMemory(*prn_name)
    Else            ; Windows NT 4.0 or earlier
        ; Get the default string from Win.ini (the registry).
        ; String will be in form "prn_name,drivername,portname".
        Define *ch_buffer = AllocateMemory(#DEFAULTPRINTERDC_BUFFERSIZE)
        If GetProfileString_("windows", "device", ",,,", *ch_buffer, #DEFAULTPRINTERDC_BUFFERSIZE) <= 0
            ProcedureReturn #Null
        EndIf
        ; Printer name precedes first "," character.
        Define prn_name.s = StringField(PeekS(*ch_buffer), 1, ",")
        FreeMemory(*ch_buffer)
        hdc = CreateDC_(#Null, @prn_name, #Null, #Null)
    EndIf

    ProcedureReturn hdc
EndProcedure

Procedure.s get_window_text(hwin)
    Define text.s
    Define length = GetWindowTextLength_(hwin) + 1
    Define *text = AllocateMemory(length*SizeOf(CHAR))
    GetWindowText_(hwin, *text, length)
    text = PeekS(*text, length)
    FreeMemory(*text)
    ProcedureReturn text
EndProcedure

Procedure get_window_size(hwin, *size.SIZE)
    Define rc.RECT
    If Not GetWindowRect_(hwin, rc)
        ProcedureReturn #False
    EndIf

    *size\cx = rc\right - rc\left
    *size\cy = rc\bottom - rc\top

    ProcedureReturn #True
EndProcedure

Procedure.s system_error_text(err_code)
    Define err_message.s
    Define perr_message, cb_message 
    cb_message = FormatMessage_(#FORMAT_MESSAGE_ALLOCATE_BUFFER | #FORMAT_MESSAGE_FROM_SYSTEM, 0, err_code, 0, @perr_message,0, 0)

    If cb_message
        err_message.s = PeekS(perr_message, cb_message)
    Else
        err_message.s = "Could not translate system error message " + Str(err_code)
    EndIf

    ProcedureReturn err_message
EndProcedure



; IDE Options = PureBasic 4.61 Beta 2 (Windows - x86)
; CursorPosition = 34
; FirstLine = 26
; Folding = -------
; EnableXP
; EnablePurifier